/*
    This class is a basic extension of the Dialog class.  It can be used
    by subclasses of Frame.  To use it, create a reference to the class,
    then instantiate an object of the class (pass 'this' in the constructor),
    and call the show() method.

    example:

    ConnectDialog1 theConnectDialog1;
    theConnectDialog1 = new ConnectDialog1(this);
    theConnectDialog1.show();

    You can add controls to AboutBox with Cafe Studio.
    (Menus can be added only to subclasses of Frame.)
 */

import java.awt.*;

public class ConnectDialog1 extends Dialog {

  //{{DECLARE_CONTROLS
    Button connectToButton;
    Button newButton;
    Button detailsButton;
    Button cancelButton;
    List conferenceList;
    Label conferenceLabel;
    Label ownerLabel;
    Label statusLabel;
    //}}
    ConnectInfo connectInfo;


  public ConnectDialog1(Frame parent,boolean modal,ConnectInfo defaultConnectSettings) {

	  super(parent, "Conferences", modal);
	  connectInfo=defaultConnectSettings;
    setup();
	  //{{INIT_CONTROLS
	  setLayout(null);
	  addNotify();
	  resize(insets().left + insets().right + 518, insets().top + insets().bottom + 268);
	  connectToButton=new Button("Connect To");
	  connectToButton.disable();
	  add(connectToButton);
	  connectToButton.reshape(insets().left + 9,insets().top + 200,117,32);
	  newButton=new Button("New");
	  add(newButton);
	  newButton.reshape(insets().left + 135,insets().top + 200,117,32);
	  detailsButton=new Button("Details...");
	  detailsButton.disable();
	  add(detailsButton);
	  detailsButton.reshape(insets().left + 261,insets().top + 200,117,32);
	  cancelButton=new Button("Cancel");
	  add(cancelButton);
	  cancelButton.reshape(insets().left + 387,insets().top + 200,117,32);
	  conferenceList=new List();
	  conferenceList.setBackground(Color.white);
	  add(conferenceList);
	  conferenceList.reshape(insets().left + 64,insets().top + 41,385,126);
	  conferenceList.addItem("conference1");
	  conferenceList.addItem("conference2");
	  conferenceList.addItem("conference3");
	  conferenceList.addItem("conference4");
	  conferenceList.addItem("conference5");
	  conferenceList.addItem("conference6");
	  conferenceList.addItem("conference7");
	  conferenceList.addItem("conference8");
	  conferenceLabel=new Label("Conference");
	  add(conferenceLabel);
	  conferenceLabel.reshape(insets().left + 63,insets().top + 24,90,16);
	  ownerLabel=new Label("Owner");
	  add(ownerLabel);
	  ownerLabel.reshape(insets().left + 209,insets().top + 24,90,16);
	  statusLabel=new Label("Status");
	  add(statusLabel);
	  statusLabel.reshape(insets().left + 349,insets().top + 24,90,16);
	  //}}

	  }

	  void setup(){

    	setResizable(false);
    }

    public void show() {
    	Rectangle bounds = getParent().bounds();
    	Rectangle abounds = bounds();

    	move(bounds.x + (bounds.width - abounds.width)/ 2,
    	     bounds.y + (bounds.height - abounds.height)/2);

    	super.show();
    }

    public synchronized void wakeUp() {
    	notify();
    }

    public boolean handleEvent(Event event) {
    	if (event.id == Event.ACTION_EVENT && event.target == detailsButton) {
    	    	clickedDetailsButton();
    	    	return true;
    	}
    	else
    	if (event.id == Event.ACTION_EVENT && event.target == connectToButton) {
    	    	clickedConnectToButton();
    	    	return true;
    	}
    	else
    	if (event.id == Event.ACTION_EVENT && event.target == newButton) {
    	    	clickedNewButton();
    	    	return true;
    	}
    	else
    	if (event.id == Event.ACTION_EVENT && event.target == conferenceList) {
    	    	doubleClickConferenceList();
    	    	return true;
    	}
    	else
    	if (event.id == Event.ACTION_EVENT && event.target == cancelButton) {
    	    	clickedCancelButton();
    	    	return true;
    	}
    	else
    	if (event.id == Event.LIST_SELECT && event.target == conferenceList) {
    	    	selChangeConferenceList(event);
    	    	return true;
    	}
    	else

    	/*if (event.id == Event.WINDOW_DESTROY) {
    	    hide();
    	    return true;
    	}*/
    	return super.handleEvent(event);
    }


  public void selChangeConferenceList(Event ev) {
      connectToButton.enable();
      detailsButton.enable();
  }
  public void clickedCancelButton() {
      dispose();

  }
  public void doubleClickConferenceList() {
      // to do: put event handler code here.
  }
  public void clickedNewButton() {
    //display_error_warning("CONNECT MODE");
    //ConnectInfo connectInfo = new ConnectInfo();
    //connectInfo=defaultConnectSettings;
    //Dialog dialog;
    //dialog= new ConnectDialog2(this,"Create new conference",true,connectInfo);
    //dialog.show();
    //return true;
  }
  public void clickedConnectToButton() {
      // to do: put event handler code here.
  }
  public void clickedDetailsButton() {
      // to do: put event handler code here.
  }
}


