//ExtendedCanvas.java

import java.awt.*;
import java.util.Vector;
import java.lang.*;

class ExtendedCanvas extends Canvas {

  public static int INITIAL_SIZE=100;
  public static int INCREMENT=100;
  int tool,objLineWidth,freehandId,firstElement,lastElement,
      newX,newY,initialX,initialY,tempId,index,stringLength;
  boolean newObject,debug,first,position,newLine;
  Color objColor;
  Color defaultBackgroundColor;
  Color backgroundColor;
  Font objFont;
  Font defaultFont;
  FontMetrics fontMetrics;
  Vector objects;
  Cursor cursor;
  JavaDrawObject current=null;
  JavaDrawObject remoteObject=null;
  JavaDrawObject point=new JavaDrawObject();
  JavaDrawObject textObject=null;
  JavaDrawObject remoteTextObject=null;
  ResultProcessor owner;





  public ExtendedCanvas(ResultProcessor temp,int toolType) {
    super();
    init(temp,toolType);
    setBackground(backgroundColor);

  }

  void init(ResultProcessor temp,int toolType){
    debug=false;
    tool = toolType;
    owner =temp;
    objects = new Vector(INITIAL_SIZE,INCREMENT);
    objColor=Color.black;
    defaultBackgroundColor=Color.white;
    backgroundColor=defaultBackgroundColor;
    newObject = false;
    objLineWidth=1;
    freehandId=0;
    firstElement=0;
    lastElement=0;
    index=0;
    position=false;
    first=true;
    cursor=null;
    newLine=false;

  }

  public void setTool(int toolType) {
    tool = toolType;
    if(tool!=JavaDrawObject.TEXT && cursor!=null){
      Cursor temp;
      temp=cursor;
      cursor=null;
      repaint(temp.baseLineX,temp.baseLineY-temp.ascent,temp.width,temp.ascent);
    }
  }

  public void setColor(Color color){
    objColor=color;
  }

  public Color getColor(){
    return objColor;
  }

  public void setBackgroundColor(Color color){
    backgroundColor=color;
    setBackground(backgroundColor);
    repaint();
  }

  public Color getBackgroundColor(){
    return backgroundColor;
  }

  public void setLineWidth(int width){
    objLineWidth=width;
  }

  public int getLineWidth(){
    return objLineWidth;
  }

  public void clear() {
    objects.removeAllElements();
    setBackground(defaultBackgroundColor);
    cursor=null;
    repaint();
  }

  public void setFont(Font font){
    objFont=font;
    fontMetrics=getFontMetrics(objFont);
  }

  public Font getFont(){
    return objFont;
  }

  public void setDefaultFont(Font font){
    defaultFont=font;
    objFont=defaultFont;
    fontMetrics=getFontMetrics(defaultFont);
  }

  public Font getDefaultFont(){
    return defaultFont;
  }


  /**
   * Delete  id JavaDrawObject on local canvas.
   *
   * @param   id      The objects unique identifier
   *
   */
  public boolean deleteObject(int id){
    boolean deleted=false;
    if(debug) System.out.println("id of object to remove: "+id);
    if(objects.isEmpty()){
      if(debug) System.out.println("objects is empty");
      return false;
    }
    else{
      if(debug) System.out.println("objects.size()="+objects.size());
      int x1=0,y1=0,x2=0,y2=0,width=0,type=0;
      for(int i=objects.size()-1;i>=0;i--){
        if(((JavaDrawObject)objects.elementAt(i)).getId()==id){
          x1=((JavaDrawObject)objects.elementAt(i)).startX;
          y1=((JavaDrawObject)objects.elementAt(i)).startY;
          x2=((JavaDrawObject)objects.elementAt(i)).endX;
          y2=((JavaDrawObject)objects.elementAt(i)).endY;
          width=((JavaDrawObject)objects.elementAt(i)).objectLineWidth;
          type=((JavaDrawObject)objects.elementAt(i)).type;
          objects.removeElementAt(i);
          deleted=true;
          if(type==JavaDrawObject.TEXT){
            cursor=null;
            repaint();
          }
          else if(type==JavaDrawObject.FREEHAND)
            repaint(x1,y1,width,width);
          else if(type==JavaDrawObject.LINE)
            repaint(Math.min(x1,x2)-width,Math.min(y1,y2)-width,Math.abs(x2-x1)+2*width,Math.abs(y2-y1)+2*width);
          else repaint(x1,y1,x2,y2);
        }
      }
      return deleted;
    }

  }

  /**
   * Add a remote JavaDrawObject to local canvas.
   *
   * @param   x1      The objects upper right corner
   * @param   y1      The objects upper right corner
   * @param   x2      Second x coordinat, used as lower left corner or width
   * @param   y2      Second y coordinat, used as lower left corner or width
   * @param   tool    Specifies object type
   * @param   id      The objects unique identifier
   * @param   RGB     The objects color
   * @param   width   The objects line width
   *
   */
  public void addObject(int x1, int y1, int x2, int y2, int tool,int id, int RGB, int width){
    remoteObject = new JavaDrawObject(tool,id,x1,y1,x2,y2,width);
    Color color=new Color(RGB);
    remoteObject.setColor(color);
    remoteObject.setId(id);
    remoteObject.setLineWidth(width);
    if(debug) System.out.println("remoteObjectId: "+remoteObject.getId());
    objects.addElement(remoteObject);
    repaint(remoteObject.startX,remoteObject.startY,remoteObject.endX,remoteObject.endY);
  }

  /**
   * Add a remote "Text" JavaDrawObject to local canvas.
   *
   * @param   x       The x coordinate for the base line of the character
   * @param   y       The y coordinate for the base line of the character
   * @param   w       The width of the character
   * @param   h       The height of the character
   * @param   size    The font size
   * @param   style   The font style
   * @param   name    The font name
   * @param   tool    Specifies object type
   * @param   id      The objects unique identifier
   * @param   charId  The characters unique identifier
   * @param   key     The value of the character
   * @param   RGB     The objects color
   *
   */
  public void addTextObject(int x, int y, int w, int h,int size,int style,String name, int type,int id,int charId,int key, int rgb ){
      int pos;
      if(debug)System.out.println("canvas.addTextObject");
      remoteTextObject = new JavaDrawObject(type,x,y,w,h);
      Color color=new Color(rgb);
      Font tempFont =new Font(name,style,size);
      remoteTextObject.setColor(color);
      remoteTextObject.setId(id);
      remoteTextObject.setCharId(charId);
      remoteTextObject.setChar(key);
      remoteTextObject.setFont(tempFont);

      pos=findElement(id,charId);

      if(charId==stringLength(id)) objects.addElement(remoteTextObject);
      else{
        for(int i=pos;i<objects.size();i++){
          if(id==((JavaDrawObject)objects.elementAt(i)).getId()){
            ((JavaDrawObject)objects.elementAt(i)).setCharId(((JavaDrawObject)objects.elementAt(i)).getCharId()+1);
            if(((JavaDrawObject)objects.elementAt(i)).startY==y) ((JavaDrawObject)objects.elementAt(i)).setStartX(((JavaDrawObject)objects.elementAt(i)).startX+w);
          }
        }
        objects.insertElementAt(remoteTextObject,pos);
      }
      if(debug) System.out.println("remoteTextObjectId: "+remoteTextObject.getId());
      repaint();//remoteTextObject.startX,remoteTextObject.startY,remoteTextObject.endX,remoteTextObject.endY);
  }

  /**
   * Handle remote backspace, delete and enter action on local canvas.
   *
   * @param   id      The objects unique identifier
   * @param   charId  The characters unique identifier
   * @param   key     The value of backspace, delete or enter
   *
   */
  public void deleteTextObject(int id, int charId, int key){
    int pos=0,width,height;
    height=fontMetrics.getHeight();
    if(debug) System.out.println("canvas.deleteTextObject");
    if(debug) System.out.println("remote id: "+id);
    if(debug) System.out.println("remote charId: "+charId);
    if(debug) System.out.println("remote key: "+key);
    if(key==127/*Delete*/){
      pos=findElement(id,charId);
      if(debug) System.out.println("pos: "+pos);
      width=((JavaDrawObject)objects.elementAt(pos)).endX;
      for(int i=pos;i<objects.size();i++){
        if(id==((JavaDrawObject)objects.elementAt(i)).getId()){
          ((JavaDrawObject)objects.elementAt(i)).setCharId(((JavaDrawObject)objects.elementAt(i)).getCharId()-1);
          if(((JavaDrawObject)objects.elementAt(i)).startY==((JavaDrawObject)objects.elementAt(pos)).startY)
            ((JavaDrawObject)objects.elementAt(i)).setStartX(((JavaDrawObject)objects.elementAt(i)).startX-width);
        }
      }
      objects.removeElementAt(pos);
      repaint();
    }
    else if(key==8/*Backspace*/){
      if(charId==stringLength(id)){
        pos=findLastElement(id);
        if(debug) System.out.println("pos: "+pos);
      }
      else{
        pos=findElement(id,charId-1);
        if(debug) System.out.println("pos: "+pos);
        width=((JavaDrawObject)objects.elementAt(pos)).endX;
        for(int i=pos;i<objects.size();i++){
          if(id==((JavaDrawObject)objects.elementAt(i)).getId()){
            ((JavaDrawObject)objects.elementAt(i)).setCharId(((JavaDrawObject)objects.elementAt(i)).getCharId()-1);
            if(((JavaDrawObject)objects.elementAt(i)).startY==((JavaDrawObject)objects.elementAt(pos)).startY)
              ((JavaDrawObject)objects.elementAt(i)).setStartX(((JavaDrawObject)objects.elementAt(i)).startX-width);
          }
        }
      }
      objects.removeElementAt(pos);
      repaint();
    }
    else if(key==99){
      pos=findElement(id,charId);
      int tempY=((JavaDrawObject)objects.elementAt(pos)).startY;
      int pos1=findElement(id,charId-1);
      int firstX=((JavaDrawObject)objects.elementAt(findFirstElement(id))).startX;
      for(int i=pos;i<objects.size();i++){
        if(((JavaDrawObject)objects.elementAt(i)).startY==tempY){
          ((JavaDrawObject)objects.elementAt(i)).setStartX(((JavaDrawObject)objects.elementAt(pos1)).startX+((JavaDrawObject)objects.elementAt(pos1)).endX-firstX+((JavaDrawObject)objects.elementAt(i)).startX);
          ((JavaDrawObject)objects.elementAt(i)).setStartY(((JavaDrawObject)objects.elementAt(pos1)).startY);
        }
        else ((JavaDrawObject)objects.elementAt(i)).setStartY(((JavaDrawObject)objects.elementAt(i)).startY-height);
      }
      repaint();
    }
    else if(key==10/*Enter*/){
      pos=findElement(id,charId);
      int newX=((JavaDrawObject)objects.elementAt(findFirstElement(id))).startX;
      int distance=((JavaDrawObject)objects.elementAt(pos)).startX-newX;
      for(int i=pos;i<objects.size();i++){
        if(id==((JavaDrawObject)objects.elementAt(i)).getId()){
          ((JavaDrawObject)objects.elementAt(i)).setStartY(((JavaDrawObject)objects.elementAt(i)).startY+height);
          if(((JavaDrawObject)objects.elementAt(i)).startY==((JavaDrawObject)objects.elementAt(pos)).startY)
            ((JavaDrawObject)objects.elementAt(i)).setStartX(((JavaDrawObject)objects.elementAt(i)).startX-distance);
        }
      }
      repaint();
    }
  }

 /*  void deleteObject(int id){
    //display_error_warning("DELETE MODE");
    if(deleteObject(id-1)) owner.decrementObjectId();
  }

  void deleteRemoteObject(int id){
    if(deleteObject(id-1));
  }
  */

  /**
   * Overrides awt.Component.mouseDown()
   *
   * @param   event   The mouse down event
   * @param   x       The mouse x position
   * @param   y       The mouse y position
   * @return  true if the event was handled
   */
  public boolean mouseDown(Event event,int x,int y) {
    if(tool==JavaDrawObject.TEXT){
      position=true;
      first=true;
      tempId=0;
      index=0;
      stringLength=0;
      initialX=x;
      initialY=y;
      newX=initialX;
      newY=initialY;
      //System.out.println("initialX: "+initialX+" initialY: "+initialY+" ascent: "+fontMetrics.getAscent()+" width: "+fontMetrics.charWidth(' '));
      cursor=new Cursor(initialX,initialY,fontMetrics.charWidth('a'),fontMetrics.getAscent(),backgroundColor);
      repaint();
      //System.out.println("MouseDown:\nPosition: "+position+"\nx="+initialX+"\ny="+initialY);
      //return true;
    }
    else if(tool==JavaDrawObject.FREEHAND){
      freehandId=owner.getObjectId();
      firstElement=objects.size();
      if(debug) System.out.println("firstElement="+firstElement);
    }
    else{
      current = new JavaDrawObject(tool,x,y);
      current.setColor(objColor);
      current.setId(owner.getObjectId());
      current.setLineWidth(objLineWidth);
      if(debug) System.out.println("currentId: "+current.getId());
      newObject = true;
    }
    return true;
  }

  /**
   * Overrides awt.Component.mouseUp()
   *
   * @param   evt     The mouse up event
   * @param   x       The mouse x position
   * @param   y       The mouse y position
   * @return  true if the event was handled
   */
  public boolean mouseUp(Event event,int x,int y) {
    if(tool==JavaDrawObject.FREEHAND){
      lastElement=objects.size()-1;
      if(debug) System.out.println("lastElement="+lastElement);
      for(int i=firstElement;i<lastElement;i++){
        sendObject((JavaDrawObject)objects.elementAt(i));
      }
    }
    else if(newObject){
      objects.addElement(current);
      sendObject(current);
      newObject = false;
    }
    return true;
  }

  /**
   * Overrides awt.Component.mouseDrag()
   *
   * @param   evt     The mouse up event
   * @param   x       The mouse x position
   * @param   y       The mouse y position
   * @return  true if the event was handled
   */
  public boolean mouseDrag(Event event,int x,int y) {
    if(tool==JavaDrawObject.FREEHAND){
      if((float)Math.abs(point.startX-x)>=(float)objLineWidth*.75 || (float)Math.abs(point.startY-y)>=(float)objLineWidth*.75){
        point = new JavaDrawObject(tool,x,y);
        point.setColor(objColor);
        point.setId(freehandId);
        point.setLineWidth(objLineWidth);
        if(debug) System.out.println("pointId: "+point.getId());
        objects.addElement(point);
        repaint(point.startX,point.startY,point.objectLineWidth,point.objectLineWidth);
      }
    }
    else if(newObject) {
      int oldX = current.endX;
      int oldY = current.endY;
      if(tool != JavaDrawObject.LINE) {
        if((event.modifiers & Event.SHIFT_MASK /* +Shift */)!=0){
          int difference=Math.abs(x-current.startX)-Math.abs(y-current.startY);
          if(difference>=0)y+=difference;
          else x+=Math.abs(difference);
        }
        if(x > current.startX) current.endX = x;
        if(y > current.startY) current.endY = y;
        int width = Math.max(oldX,current.endX) - current.startX + 1;
        int height = Math.max(oldY,current.endY) - current.startY + 1;
        repaint(current.startX,current.startY,width,height);
      }
      else{
        current.endX = x;
        current.endY = y;
        int startX = Math.min(Math.min(current.startX,current.endX),oldX);
        int startY = Math.min(Math.min(current.startY,current.endY),oldY);
        int endX = Math.max(Math.max(current.startX,current.endX),oldX);
        int endY = Math.max(Math.max(current.startY,current.endY),oldY);
        repaint(startX-objLineWidth,startY-objLineWidth,endX-startX+objLineWidth*2,endY-startY+objLineWidth*2);
      }
    }
    return true;
  }

  /**
   * Overrides awt.Component.keyUp() to handle upper and lower case
   * character input, Digits, Enter, Backspace, right and left arrow.
   *
   * @param   event   The mouse enter event
   * @param   key     The key value
   * @return  true if the event was handled
   */
  public boolean keyUp(Event event,int key){

    if(tool==JavaDrawObject.TEXT && position){
      if(first){
        tempId=owner.getObjectId();
        first=false;
      }
      if(key==Event.LEFT /*Left arrow*/){
          stepLeft();
          repaint();
          return true;
      }
      else if(key==Event.RIGHT /*Right arrow*/){
          stepRight();
          repaint();
          return true;
      }
      if(key==Event.UP /* Up arrow*/){
          return true;
      }
      else if(key==Event.DOWN /*Down arrow*/){
        return true;
      }
      else if(key==10 /*Enter*/ ){
          enter();
          repaint();
          newLine=true;
          return true;
      }
      else if(key==8 /*Backspace*/){
          removeChar(key);
          repaint();
          return true;
      }
      else if(key==127/*127Delete*/){
          deleteChar(key);
          repaint();
          return true;
      }
      else if(Character.isLowerCase((char)key) || //Digits & Characters
                Character.isUpperCase((char)key) ||
                Character.isDigit((char)key) ||
                Character.isSpace((char)key)){
          addChar(key,newLine);
          repaint();
          return true;
                }
    }/*else if(key==127 /*Delete){
        deleteObject(owner.getLastObjectId());
        owner.sendDeleteData(owner.getLastObjectId());
        return true;
    }*/
    return false;
  }

    /**
     * Finds the vector element described by Id and charId.
     *
     * @param   id      Id of the JavaDrawObject
     * @param   charId  Id of the character.
     * @return  pos     Return the vector position of the element
     *                  described by id and charId
     */
    int findElement(int id, int charId){
      int pos=-1;
      for(int i=0;i<objects.size();i++){
        if(id==((JavaDrawObject)objects.elementAt(i)).getId()){
          if(charId==((JavaDrawObject)objects.elementAt(i)).getCharId()) pos=i;
        }
      }
      if(debug)System.out.println("pos: "+pos);
      return pos;
    }

    /**
     * Finds the first vector element whith the specific Id.
     *
     * @param   id      Id of the JavaDrawObject
     * @return  pos     Return the vector position of the element
     *                  described by id.
     */
    int findFirstElement(int id){
      for(int i=0;i<objects.size();i++){
        if(id==((JavaDrawObject)objects.elementAt(i)).getId()) return i;
      }
      return 0;
    }

    /**
     * Finds the the last vector element whith the specific Id.
     *
     * @param   id      Id of the JavaDrawObject
     * @return  pos     Return the vector position of the element
     *                  described by id.
     */
    int findLastElement(int id){
      int pos=0;
      for(int i=0;i<objects.size();i++){
        if(id==((JavaDrawObject)objects.elementAt(i)).getId()) pos=i;
      }
      if(debug)System.out.println("pos: "+pos);
      return pos;
    }

    /**
     * Finds the # of elements with the specified id.
     *
     * @param   id      Id of the JavaDrawObject
     * @return  length  Returns the # of elements with the
     *                  specified id
     */
    int stringLength(int id){
      int length=0;
      for(int i=0;i<objects.size();i++){
        if(id==((JavaDrawObject)objects.elementAt(i)).getId()) length+=1;
      }
      return length;
    }


    /**
     * Moves cursor one step to the right.
     */
    void stepRight(){
      if(stringLength==0) return;
      if(index<stringLength) index++;
      int pos=findElement(tempId,index);
      if(pos>=0) cursor=new Cursor(((JavaDrawObject)objects.elementAt(pos)).startX,((JavaDrawObject)objects.elementAt(pos)).startY,((JavaDrawObject)objects.elementAt(pos)).endX,((JavaDrawObject)objects.elementAt(pos)).endY,backgroundColor);
      else{
        pos=findLastElement(tempId);
        cursor=new Cursor(((JavaDrawObject)objects.elementAt(pos)).startX+fontMetrics.charWidth('a'),((JavaDrawObject)objects.elementAt(pos)).startY,((JavaDrawObject)objects.elementAt(pos)).endX,((JavaDrawObject)objects.elementAt(pos)).endY,backgroundColor);
      }
      if(debug)System.out.println("->");
      if(debug)System.out.println("pos: "+pos+" index: "+index+" stringLength: "+stringLength+" tempId: "+tempId);
    }

    /**
     * Moves cursor one step to the left.
     */
    void stepLeft(){
      if(index==0) return;
      index--;
      int pos=findElement(tempId,index);
      cursor=new Cursor(((JavaDrawObject)objects.elementAt(pos)).startX,((JavaDrawObject)objects.elementAt(pos)).startY,((JavaDrawObject)objects.elementAt(pos)).endX,((JavaDrawObject)objects.elementAt(pos)).endY,backgroundColor);
      if(debug)System.out.println("<-");
      //if(debug)System.out.println("pos: "+pos+" index: "+index+" stringLength: "+stringLength+" tempId: "+tempId);
    }

    /**
     * Handels the function of pressed enter key, in text mode.
     */
    void enter(){
      int pos=0,height=0,distance=0;
      height=fontMetrics.getHeight();
      if(index==0){
        newX=initialX;
        newY=initialY;
      }
      else if(stringLength==0){
        newY+=height;
        //newX=initialX;
        cursor=new Cursor(newX,newY,fontMetrics.charWidth('a'),fontMetrics.getAscent(),backgroundColor);
      }
      else if(index==stringLength){
        newX=initialX;
        pos=findLastElement(tempId);
        newY=((JavaDrawObject)objects.elementAt(pos)).startY+height;
        cursor=new Cursor(newX,newY,fontMetrics.charWidth('a'),fontMetrics.getAscent(),backgroundColor);
      }
      else{
        newX=initialX;
        pos=findElement(tempId,index);
        distance=((JavaDrawObject)objects.elementAt(pos)).startX-initialX;
        for(int i=pos;i<objects.size();i++){
          if(tempId==((JavaDrawObject)objects.elementAt(i)).getId()){
            ((JavaDrawObject)objects.elementAt(i)).setStartY(((JavaDrawObject)objects.elementAt(i)).startY+height);
            if(((JavaDrawObject)objects.elementAt(i)).startY==((JavaDrawObject)objects.elementAt(pos)).startY)
              ((JavaDrawObject)objects.elementAt(i)).setStartX(((JavaDrawObject)objects.elementAt(i)).startX-distance);
          }
        }
        cursor=new Cursor(((JavaDrawObject)objects.elementAt(pos)).startX,((JavaDrawObject)objects.elementAt(pos)).startY,((JavaDrawObject)objects.elementAt(pos)).endX,((JavaDrawObject)objects.elementAt(pos)).endY,backgroundColor);
        sendCtrlChar(tempId,index,10/*enter*/);

      }
    }

    /**
     * Handels the function of pressed delete key, in text mode.
     * @param key     key pressed
     */
    void deleteChar(int key){
      int pos=0,width=0,cursorPos=0;
      if(stringLength==0 || stringLength==index)return;// Empty String or behind last letter
      pos=findElement(tempId,index);
      if(index==0 && stringLength==1)// first letter
        cursor =new Cursor(initialX,initialY,fontMetrics.charWidth('a'),fontMetrics.getAscent(),backgroundColor);
      else if(pos==findLastElement(tempId)){// last letter
        cursorPos=pos;
        cursor=new Cursor(((JavaDrawObject)objects.elementAt(cursorPos)).startX,((JavaDrawObject)objects.elementAt(cursorPos)).startY,((JavaDrawObject)objects.elementAt(cursorPos)).endX,((JavaDrawObject)objects.elementAt(cursorPos)).endY,backgroundColor);
      }
      else{
        cursorPos=findElement(tempId,index+1);
        if(cursorPos==-1) cursorPos=pos;
        width=((JavaDrawObject)objects.elementAt(pos)).endX;
        for(int i=pos;i<objects.size();i++){
          if(tempId==((JavaDrawObject)objects.elementAt(i)).getId()){
            ((JavaDrawObject)objects.elementAt(i)).setCharId(((JavaDrawObject)objects.elementAt(i)).getCharId()-1);
            if(((JavaDrawObject)objects.elementAt(i)).startY==((JavaDrawObject)objects.elementAt(pos)).startY)
              ((JavaDrawObject)objects.elementAt(i)).setStartX(((JavaDrawObject)objects.elementAt(i)).startX-width);
          }
        }
        cursor=new Cursor(((JavaDrawObject)objects.elementAt(cursorPos)).startX,((JavaDrawObject)objects.elementAt(cursorPos)).startY,((JavaDrawObject)objects.elementAt(cursorPos)).endX,((JavaDrawObject)objects.elementAt(cursorPos)).endY,backgroundColor);
      }
      objects.removeElementAt(pos);
      sendCtrlChar(tempId,index,key);
      //sendDeleteChar(tempId,index);
      stringLength--;
      if(debug)System.out.println("pos: "+pos+" index: "+index+" stringLength: "+stringLength+" tempId: "+tempId);
    }

    /**
     * Handels the function of pressed backspace key, in text mode.
     * @param key     key pressed
     */
    void removeChar(int key){
      int pos=0,width=0,height=0,cursorPos=0;
      height=fontMetrics.getHeight();
      pos=findElement(tempId,index);
      if(index==0) return;

      else if(cursor.baseLineX==initialX){
        if(index==stringLength){
          cursorPos=findLastElement(tempId);
          cursor=new Cursor(((JavaDrawObject)objects.elementAt(cursorPos)).startX+((JavaDrawObject)objects.elementAt(cursorPos)).endX,((JavaDrawObject)objects.elementAt(cursorPos)).startY,fontMetrics.charWidth('a'),fontMetrics.getAscent(),backgroundColor);
          return;
        }
        else{
          int tempY=((JavaDrawObject)objects.elementAt(pos)).startY;
          int pos1=findElement(tempId,index-1);
          for(int i=pos;i<objects.size();i++){
            if(((JavaDrawObject)objects.elementAt(i)).startY==tempY){
              ((JavaDrawObject)objects.elementAt(i)).setStartX(((JavaDrawObject)objects.elementAt(pos1)).startX+((JavaDrawObject)objects.elementAt(pos1)).endX-initialX+((JavaDrawObject)objects.elementAt(i)).startX);
              ((JavaDrawObject)objects.elementAt(i)).setStartY(((JavaDrawObject)objects.elementAt(pos1)).startY);
            }
            else{
              ((JavaDrawObject)objects.elementAt(i)).setStartY(((JavaDrawObject)objects.elementAt(i)).startY-height);
            }
          }
          cursorPos=pos;
          cursor=new Cursor(((JavaDrawObject)objects.elementAt(cursorPos)).startX,((JavaDrawObject)objects.elementAt(cursorPos)).startY,((JavaDrawObject)objects.elementAt(cursorPos)).endX,((JavaDrawObject)objects.elementAt(cursorPos)).endY,backgroundColor);
          sendCtrlChar(tempId,index,99 /*Remove line*/);

          return;
        }
      }
      else if(index==stringLength){
        pos=findLastElement(tempId);
        cursorPos=pos;
        cursor=new Cursor(((JavaDrawObject)objects.elementAt(cursorPos)).startX,((JavaDrawObject)objects.elementAt(cursorPos)).startY,fontMetrics.charWidth('a'),fontMetrics.getAscent(),backgroundColor);
        objects.removeElementAt(pos);
        sendCtrlChar(tempId,index,key /*Backspace*/);
        index--;
        stringLength--;
        return;
      }
      else{
          System.out.println("DataData");
          pos=findElement(tempId,index-1);
          cursorPos=pos;
          width=((JavaDrawObject)objects.elementAt(pos)).endX;
          for(int i=pos;i<objects.size();i++){
            if(tempId==((JavaDrawObject)objects.elementAt(i)).getId()){
              ((JavaDrawObject)objects.elementAt(i)).setCharId(((JavaDrawObject)objects.elementAt(i)).getCharId()-1);
              if(((JavaDrawObject)objects.elementAt(i)).startY==((JavaDrawObject)objects.elementAt(pos)).startY)
                ((JavaDrawObject)objects.elementAt(i)).setStartX(((JavaDrawObject)objects.elementAt(i)).startX-width);
            }
          }
          objects.removeElementAt(pos);
          sendCtrlChar(tempId,index,key /*Backspace*/);
          index--;
          stringLength--;
          cursor=new Cursor(((JavaDrawObject)objects.elementAt(cursorPos)).startX,((JavaDrawObject)objects.elementAt(cursorPos)).startY,((JavaDrawObject)objects.elementAt(cursorPos)).endX,((JavaDrawObject)objects.elementAt(cursorPos)).endY,backgroundColor);
          return;
        }
    }

    /**
     * Adds a new character or digit to the screen, in text mode.
     * @param key     key pressed
     * @param enter   signals that the enter key has ben pressed.
     */
    void addChar(int key,boolean enter){
      int pos=0,width=0;
      width=fontMetrics.charWidth(key);
      if(enter){
        newX=initialX;
        newLine=false;
      }

      else if(index==0 && stringLength==0){
        newX=initialX;
        newY=initialY;
      }
      else if(index==stringLength){
        pos=findLastElement(tempId);
        newX=((JavaDrawObject)objects.elementAt(pos)).startX+((JavaDrawObject)objects.elementAt(pos)).endX;
        newY=((JavaDrawObject)objects.elementAt(pos)).startY;
      }
      else{
        pos=findElement(tempId,index);
        newX=((JavaDrawObject)objects.elementAt(pos)).startX;
        newY=((JavaDrawObject)objects.elementAt(pos)).startY;
      }
      textObject = new JavaDrawObject(tool,newX,newY,width,fontMetrics.getAscent());
      textObject.setColor(objColor);
      textObject.setId(tempId);
      textObject.setCharId(index);
      textObject.setChar(key);
      textObject.setFont(objFont);

      if(index==stringLength/*last*/){
        objects.addElement(textObject);
        sendObject(textObject);
        cursor=new Cursor(newX+width,newY,fontMetrics.charWidth('a'),fontMetrics.getAscent(),backgroundColor);
      }
      else{
        for(int i=pos;i<objects.size();i++){
          if(tempId==((JavaDrawObject)objects.elementAt(i)).getId()){
            ((JavaDrawObject)objects.elementAt(i)).setCharId(((JavaDrawObject)objects.elementAt(i)).getCharId()+1);
            if(((JavaDrawObject)objects.elementAt(i)).startY==((JavaDrawObject)objects.elementAt(pos)).startY)
                ((JavaDrawObject)objects.elementAt(i)).setStartX(((JavaDrawObject)objects.elementAt(i)).startX+width);
          }
        }
        objects.insertElementAt(textObject,pos);
        sendObject(textObject);
        pos=findElement(tempId,index+1);
        cursor=new Cursor(((JavaDrawObject)objects.elementAt(pos)).startX,((JavaDrawObject)objects.elementAt(pos)).startY,((JavaDrawObject)objects.elementAt(pos)).endX,((JavaDrawObject)objects.elementAt(pos)).endY,backgroundColor);
      }
      index++;
      stringLength++;



      if(debug){
        /*for(int i=0;i<objects.size();i++){
          System.out.println("\nobjects.elementAt["+i+"]:");
          System.out.println("\tID: "+((JavaDrawObject)objects.elementAt(i)).getId());
          System.out.println("\tCharID: "+((JavaDrawObject)objects.elementAt(i)).getCharId());
          System.out.println("\tChar: "+((JavaDrawObject)objects.elementAt(i)).getChar());
          System.out.println("\tstartX: "+((JavaDrawObject)objects.elementAt(i)).startX);
          System.out.println("\tstartY: "+((JavaDrawObject)objects.elementAt(i)).startY);
          System.out.println("\tendX: "+((JavaDrawObject)objects.elementAt(i)).endX);
          System.out.println("\tendY: "+((JavaDrawObject)objects.elementAt(i)).endY);
        }*/
      }
      if(debug)System.out.println("pos: "+pos+" index: "+index+" stringLength: "+stringLength+" tempId: "+tempId);

    }


  /**
   * Overrides awt.Component.paint() to paint the current JavaDrawObject.
   *
   * @param     obj     The JavaDrawObject to send
   */
  void sendObject(JavaDrawObject obj){
    if(owner.isConnected()){
      if(tool==JavaDrawObject.TEXT){
        int size=((Font)(obj.objectFont)).getSize();
        int style=((Font)(obj.objectFont)).getStyle();
        String name=((Font)(obj.objectFont)).getName();
        owner.sendTextData(obj.startX,obj.startY,obj.endX,obj.endY,size,style,name,obj.type,obj.getId(),obj.getCharId(),(int)obj.character,(int)(obj.getColor()).getRGB());
      }
      else owner.sendPenData(obj.startX,obj.startY,obj.endX,obj.endY,obj.type,obj.getId(),(int)(obj.getColor()).getRGB(),obj.getLineWidth());
    }
  }

  /**
   * Overrides awt.Component.paint() to paint the current JavaDrawObject.
   *
   * @param     obj     The JavaDrawObject to send
   */
  void sendCtrlChar(int id, int charId, int key){
    if(debug)System.out.println("sendDeleteChar");
    if(debug)System.out.println("id"+id);
     if(debug)System.out.println("charId"+charId);
     owner.sendCtrlCharData(id, charId,key);
     //owner.sendDelCharData(id, charId);
  }

  /**
   * Overrides awt.Component.paint() to paint the current JavaDrawObject.
   *
   * @param     g     The Graphics in which to draw
   */
  public void paint(Graphics g) {
    int numObjects = objects.size();
    for(int i=0;i<numObjects;++i) {
      JavaDrawObject obj = (JavaDrawObject) objects.elementAt(i);
      obj.draw(g);
    }
    if(newObject) current.draw(g);
    if(cursor!=null)cursor.draw(g);

  }


}// End of class ExtendedCanvas

class Cursor{
  int baseLineX,baseLineY,ascent,width;
  Color color;

  public Cursor(int x,int y,int w,int h,Color c){
    baseLineX=x;
    baseLineY=y;
    ascent=h;
    width=w;
    color=c;
  }
  public void draw(Graphics g){
    g.setXORMode(color);
    //g.setColor(color);
    g.fillRect(baseLineX,baseLineY-ascent,width,ascent);
    g.setPaintMode();
  }
}

