
// Coded by anders sandstrom 1997
// original design by: Dee M. Patel and C. Edward Chow 12/23/96
// Free to use for education and research purpose
// Acknowledge the authors in the code you develop with this
// Given us your feedback at chow@quandary.uccs.edu
//

import java.awt.*;
import java.util.*;
import java.net.*;
import java.io.*;

////////////////////////////////////////////////////////////////////////////////////////////////
class Server {
  String ipinfo = null;
  ISList islist;

  public Server(){
  }

  public static void main ( String argv[] ) {
    int portnum = -1; // default

    if (argv.length < 1) {
      usage();
      portnum = 9005;
      System.err.println("Using port " + portnum + " by default." );
    }
    else{
      portnum = new Integer(argv[0]).intValue();
      System.err.println("Using port " + portnum);
    }
    Server isf = new Server();
    isf.rISFrame(portnum);
  }

  public void rISFrame(int portn){
    InetAddress localAddress = null;
    try{
      localAddress = InetAddress.getLocalHost();
      ipinfo = "IP ADDRESS: "+localAddress.toString() +" PORT NUMBER: "+Integer.toString(portn);
    }
    catch (UnknownHostException e){
      System.out.println("ERROR: Exiting Server");
      System.exit(0);
    }

    ISList isl = new ISList();
    islist = isl;
    ISecretary is = new ISecretary(portn);
    is.updateisf(this);
  }

  public ISList getisl(){
    return islist;
  }



  static void usage(){
    System.err.println ("Usage: java Server -port portnum" );
  }
}

//////////////////////////////////////////////////////////////////////////////////////////////////
class ISecretary extends NetServer
{
  Server isf = null;
  ISList  isl = null;
  StoreHashtable sht;

  ISecretary( int portnum){
    super ( portnum );
    sht = new StoreHashtable();
  }

  public void updateisf(Server isframe){
    isf = isframe;
  }

  protected void createClientPort( Socket socket ){
    isl = isf.getisl();

    if(isf == null || isl == null){
      System.out.println("createClientPort: isf or isl is null");
      return;
    }

    try{
      new ClientPort ( this, socket, isf, isl, sht);
    	      //System.out.println ("SOCKET "+socket );
    }
    catch( IOException ioe ){
    }
  }

  static void usage(){
    System.err.println ("Usage: java Server -port portnum" );
    System.exit( -1 );
  }
}
////////////////////////////////////////////////////////////////////////////////////////////////

class ClientPort extends NetPortToClient
{
    static int socket_num = 0;
    static Socket tempsocket[] = new Socket[50];
    static Server isf = null;
    static ISList  isl = null;
    static boolean opened_frame = false;
    Socket tempsocket2 = null;
    StoreHashtable sht;
    ISecretary myserver;
    String conf_list;

    public ClientPort( ISecretary server,
                       Socket socket,
                       Server temp_isf,
                       ISList temp_isl,
                       StoreHashtable temp_sht
                     ) throws IOException
    {
        super( server, socket );
        isf = temp_isf;
        isl = temp_isl;
        sht = temp_sht;
        myserver = server;
        start(); // as a thread
        boolean socket_saved = false;

        add_client_ip(socket);

        tempsocket2 = socket;
    	     // System.out.println ("TEMPSOCKET "+tempsocket );
    }

    synchronized protected void readInput() throws IOException
    {
        String strarray[] = new String[20];
        boolean owner = false;
        String inputdata = null;

        try
        {
            inputdata = readLine();
        }
        catch ( IOException ioe )
        {
            error ("Error reading from the client.\nClient probably disconnected" );
        }
        if(inputdata == null)
        {
            //System.out.println("readInput: inputdata is null");
            return;
        }

        if(sht != null)
        {
            Date date = new Date();
            long currtime = date.getTime();

            Object temp_obj = sht.register_revoke_time.get(tempsocket2);

            if(temp_obj != null)
            {
                RevokeInfo record = (RevokeInfo) temp_obj;
                //System.out.println (record.rtime+" "+currtime);
                if((record.rtime - currtime) > 0)
                {
                    return;
                }
                else
                {
                    sht.register_revoke_time.remove(record.socket);
                    sht.update_revoke_time(sht.register_revoke_time);
                }
            }
        }

        //System.out.println(tempsocket2);
        StringTokenizer inputbufferST =
            new StringTokenizer(inputdata, " ");
        int index = 0;
        while(inputbufferST.hasMoreTokens())
        {
            strarray[index] = inputbufferST.nextToken();
            index++;
        }

        if (strarray[2].compareTo("close") == 0)
        {
            System.out.println("Close "+inputdata);
            delete_client_info(strarray[1], strarray[3]);
        }
        else if (strarray[2].compareTo("Connect") == 0)
        {
            String client_log_id =
                search_client_name(strarray[1]);
            //System.out.println("FOUND ID "+client_log_id);
            //show_client_info();
            if(client_log_id != null)
            {
                delete_client_ip(strarray[3]);
                process("Disconnect", tempsocket2);
            }
            else
            {
                //show_client_conf();
                String tmpnm = search_client_conf(strarray[0]);
                if(tmpnm == null)
                {
                    add_client_conf(strarray[0], strarray[1]);
                    //show_client_conf();
                }
                else
                {
                    //Do not add in conf hashtable.
                    //Someone else is the owner.
                    //process("ERROR: Not owner", tempsocket2);
                }

                add_client_info(strarray[1], //id_name
                                strarray[0], //conference
                                strarray[3], //ip address
                                "-1" //resume time
                               );
            }
        }
        else if (strarray[2].compareTo("Revoke") == 0 ||
                 strarray[2].compareTo("Suspend") == 0)
        {
            String owner_name = search_client_conf(strarray[0]);
            if(owner_name.compareTo(strarray[1]) != 0)
            {
                //process("ERROR: Not owner", tempsocket2);
                System.out.println("ERROR: Not owner");
            }
            else if (strarray[5].length() == 0 ||
                     strarray[4].length() == 0)
            {
                System.out.println("ERROR: ID or TIME specified is wrong");
            }
            else
            {
                revoke_client(strarray[2], // "Revoke" or "Suspend"
                              strarray[5], //id_name
                              strarray[4] //resume time
                             );
            }
        }
        else
        {
            send_to_clients(strarray, inputdata, false);
        }
    }

    synchronized public void show_client_info()
    {
        Enumeration ipelement = sht.register_client_info.elements();

        for(;ipelement.hasMoreElements();)
        {
            Object temp_element = ipelement.nextElement();

            if(temp_element != null)
            {
                ClientInfo record = (ClientInfo) temp_element;
            }
        }
    }

    synchronized public void show_client_ip()
    {
        Enumeration ipelement = sht.register_client_ip.elements();

        for(;ipelement.hasMoreElements();)
        {
            Object temp_element = ipelement.nextElement();

            if(temp_element != null)
            {
                ClientIP record = (ClientIP) temp_element;
                //System.out.println("show_client_ip: "+record.ipkey);
            }
        }
    }

    synchronized public void show_client_conf()
    {
        Enumeration ipelement = sht.register_client_conf.elements();

        for(;ipelement.hasMoreElements();)
        {
            Object temp_element = ipelement.nextElement();

            if(temp_element != null)
            {
                ClientConfName record = (ClientConfName) temp_element;
            }
        }
    }

    synchronized public void send_to_clients(String[] strarray,
                                String inputd,
                                boolean broad_cast)
    {

        Enumeration ipelement = sht.register_client_info.elements();

        for(;ipelement.hasMoreElements();)
        {
            Object temp_element = ipelement.nextElement();

            if(temp_element != null)
            {
                Date date = new Date();
                long currtime = date.getTime();
                //System.out.println("TIME ms "+currtime);
                ClientInfo record = (ClientInfo) temp_element;
                if(broad_cast)
                {
                    process(inputd, record.socket);
                }
                else if((strarray[0].compareTo(record.conference) == 0) &&
                        (strarray[1].compareTo(record.logid) != 0) &&
                        ((record.rtime - currtime) <= 0)
                       )
                {
                    //System.out.println("PROCESS "+record.socket);
                    process(inputd, record.socket);
                }
            }
        }
    }

    synchronized public void add_client_info(String id,
                                String conf,
                                String ipadd,
                                String tm
                               )
    {
        if(id == null)
        {
            System.out.println("add_client_info: id is null");
            return;
        }
        Socket socket = null;
        show_client_ip();
        socket = search_client_ip(ipadd);
        if (socket == null)
        {
            System.out.println("add_client_info: socket is null");
            //Can not add client to list
            return;
        }
        if(id != null)
        {
            ClientInfo clientinfo =
                new ClientInfo(id, conf, socket, tm);
            sht.register_client_info.put(id, clientinfo);
            sht.update_client_info(sht.register_client_info);
            conf_list = isl.ShowList(sht);
            //System.out.println(conf_list);
            send_to_clients(null, conf_list, true);
            //System.out.println(conf_list);
        }
    }

    synchronized public void add_client_conf(String confk, String cname)
    {
        if ((confk != null) || (cname != null))
        {
            //System.out.println("ADD VALUE: "+confk+ " "+cname);
            ClientConfName clientconfname =
                new ClientConfName(confk, cname);
            sht.register_client_conf.put(confk, clientconfname);
            sht.update_client_conf(sht.register_client_conf);
        }
    }

    public void add_client_ip(Socket socket)
    {
        String ipkey = null;
        InetAddress add;
        int port;
        add = socket.getInetAddress();
        port = socket.getPort();

        ipkey = add.toString()+
                       ":"+
                       Integer.toString(port);
        if (ipkey != null)
        {
            ClientIP clientip = new ClientIP(ipkey, socket);
            sht.register_client_ip.put(ipkey, clientip);
            sht.update_client_ip(sht.register_client_ip);
        }
    }

    public void add_revoke_time(Socket socket, long time)
    {
        if (socket != null)
        {
            RevokeInfo revokeinfo = new RevokeInfo(socket, time);
            sht.register_revoke_time.put(socket, revokeinfo);
            //sht.update_revoke_time(sht.register_revoke_time);
        }
    }

    public Socket search_client_ip(String ipkey)
    {
        if(ipkey == null)
        {
            System.out.println("search_client_ip: ipkey is null");
            return null;
        }
        Object object = sht.register_client_ip.get(ipkey);

        if(object != null)
        {
            ClientIP record = (ClientIP) object;
            return record.socket;
        }
        System.out.println("search_client_ip: object is null");
        return null;
    }

    public void revoke_client(String command, String logid, String rt)
    {
        System.out.println(command+" "+logid+" "+rt);
        Date date = new Date();
        long currtime = date.getTime();
        long revoke_time = -1;

        revoke_time = (new Long(rt).longValue() * 60 * 1000) + currtime;

        if(logid == null)
        {
            System.out.println("revoke_client: logid is null");
            return;
        }

        Object object = sht.register_client_info.get(logid);

        if(object != null)
        {
            //Date date = new Date();
            //long currtime = date.getTime();
            ClientInfo record = (ClientInfo) object;
            if (command.compareTo("Suspend") == 0)
            {
                record.rtime = revoke_time;
                sht.register_client_info.put(logid, record);
                sht.update_client_info(sht.register_client_info);
                System.out.println(record.rtime);
            }
            Object temp_obj =
                     sht.register_revoke_time.get(record.socket);

            if(temp_obj != null)
            {
                sht.register_revoke_time.remove(record.socket);
            }
            add_revoke_time(record.socket, revoke_time);
            sht.update_revoke_time(sht.register_revoke_time);
        }
        return;
    }

    public String search_client_name(String name)
    {
        //System.out.println("SEARCH "+name);
        if(name == null)
        {
            //System.out.println("search_client_name: name is null");
            return null;
        }
        Object object = sht.register_client_info.get(name);

        if(object != null)
        {
            ClientInfo record = (ClientInfo) object;
            return record.logid;
        }
        //System.out.println("search_client_name: object is null");
        return null;
    }

    public String search_client_conf(String confk)
    {
        //System.out.println("SEARCH CONF "+confk);
        if(confk == null)
        {
            //System.out.println("search_client_conf: confk is null");
            return null;
        }
        Object object = sht.register_client_conf.get(confk);

        if(object != null)
        {
            ClientConfName record = (ClientConfName) object;
            return record.clientname;
        }
        //System.out.println("search_client_conf: object is null");
        return null;
    }

    public void delete_client_ip(String ipkey)
    {
        sht.register_client_ip.remove(ipkey);
        sht.update_client_ip(sht.register_client_ip);
    }

    public void delete_client_info(String ipkey, String ipadd)
    {
        if(ipkey == null || ipadd == null)
        {
            System.out.println("delete_client_info: ipkey or ipadd is null");
            return;
        }
        sht.register_client_info.remove(ipkey);
        sht.update_client_info(sht.register_client_info);
        delete_client_ip(ipadd);
        conf_list = isl.ShowList(sht);
        //System.out.println(conf_list);
        send_to_clients(null, conf_list, true);
    }



    private synchronized void process( String inputdata,
                                       Socket socket )
    {
        try {
                inputdata = inputdata.concat("\n");
                //System.out.print ("WRITE: "+inputdata+ " "+socket);
                DataOutputStream dos =
                    new DataOutputStream(socket.getOutputStream());
                dos.writeBytes(inputdata);
		dos.flush();
        }
        catch(IOException e)
        {
            System.out.println("ERROR: Writing");
        }
    }
}
/////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////////

class ISList extends List
{
    Enumeration ip;
    Enumeration skt;
    String item = null;
    StoreHashtable sht;

    public ISList()
    {
        super(4, false);
    }

    public String ShowList(StoreHashtable temp_sht)
    {

        String printList = "User" + "\t\t" + "Conference\n"+"----\t\t----------\n";// + " ";
        String conf_list = "List" + " " + "ALL" + " ";
        sht = temp_sht;
        if(sht == null || sht.register_client_info == null)
        {
            System.out.println("ShowList: sht or register_client_info is null");
            return null;
        }
        clear();
        ip = sht.register_client_info.keys();
        skt = sht.register_client_info.elements();

        for(;ip.hasMoreElements();)
        {
            Object temp_obj = ip.nextElement();
            Object temp_element = skt.nextElement();

            item = (String) temp_obj;
            //System.out.println(item);
            addItem(item);

            if(temp_element != null)
            {
                ClientInfo record = (ClientInfo) temp_element;
                printList = printList+item+"\t\t"+record.conference+"\n";
                conf_list = conf_list+item+" "+record.conference+" ";
            }
        }
        System.out.println(printList);
        return conf_list+"\n";
    }

}

class StoreHashtable
{
    Hashtable register_client_ip = new Hashtable();
    Hashtable register_client_info = new Hashtable();
    Hashtable register_client_conf = new Hashtable();
    Hashtable register_revoke_time = new Hashtable();

    public void update_client_ip(Hashtable h)
    {
        register_client_ip = h;
    }

    public void update_client_info(Hashtable h)
    {
        register_client_info = h;
    }

    public void update_client_conf(Hashtable h)
    {
        register_client_conf = h;
    }

    public void update_revoke_time(Hashtable h)
    {
        register_revoke_time = h;
    }
}


class ClientInfo
{
    String logid;
    String conference;
    Socket socket;
    long   rtime;

    public ClientInfo(String id,
                           String conf,
                           Socket skt,
                           String r_time
                          )
    {
	logid = id;
	conference = conf;
	socket = skt;
        rtime = new Long(r_time).longValue();
    }
}

class ClientIP
{
    String ipkey;
    Socket socket;
    public ClientIP(String ipk, Socket skt)
    {
	ipkey = ipk;
	socket = skt;
    }
}

class ClientConfName
{
    String confkey;
    String clientname;
    public ClientConfName(String conf_key, String c_name)
    {
        confkey = conf_key;
        clientname = c_name;
    }
}

class RevokeInfo
{
    Socket socket;
    long   rtime;

    public RevokeInfo(Socket skt, long r_time)
    {
	socket = skt;
        rtime = r_time;
    }
}

