/**
 *  Zodega.java 1997 beta
 *  Author: Anders Sandstrom
 *
 *  This version of Zodega is based on javadraw by Dee Patel.
 *
 *  Zodega is a java version of the distributed drawing tool Sharedraw
 *  developed by Dr. Edward Chow professor at University of Colorado in
 *  Colorado Springs.
 *
 */

// Import
import java.awt.*;
import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;
import easy.*;
import imgbtn.*;
//import extended.*;
//import dialogs.*;

/**
 *  Zodega - A distributed Drawingtool
 *
 *  @author Anders Sandstrom 1997
 */

public class Zodega extends Frame implements ResultProcessor {



  // Method Declarations

  /**
   *  main() - Launches javadraw, an instance of the Zodega class.
   *  main ignores command line arguments.
   */

  public static void main(String args[]){
    Zodega zodega=new Zodega();
  }




  /**
   *  Constructs an instance of Zodega.
   */

  // Method Zodega()
  public Zodega(){
    super("Beta");
    setup();
    pack();
    resize(screenWidth,screenHeight);
    show();
    init();
  }

  // Method setup()
  void setup(){
    setLayout(new BorderLayout());
    setBackground(Color.lightGray);
    setupFonts();
    setupMenuBar();
    setMenuBar(menuBar);
    setupButtonBar();
    setupStatusBar();
    setupToolButtonBar();
    add("Center",canvas);
  }


  /**
   * Overrides awt.Component.handleEvent() to rearm the button if the mouse is
   * down, or hilite it if a special OVER image and/or border was set.
   *
   * @param   evt     The mouse enter event
   * @param   x       The mouse x position
   * @param   y       The mouse y position
   * @return  true if the event was handled
   */
  public boolean handleEvent(Event event){
    if(event.id==Event.WINDOW_DESTROY) exit();
    return super.handleEvent(event);
  }

  /**
   * Overrides awt.Component.mouseEnter() to rearm the button if the mouse is
   * down, or hilite it if a special OVER image and/or border was set.
   *
   * @param   evt     The mouse enter event
   * @param   x       The mouse x position
   * @param   y       The mouse y position
   * @return  true if the event was handled
   */
  public boolean mouseEnter( Event evt, int x, int y ) {
    //if(debug) System.out.println("evt.target: "+(evt.target).toString());
    if(evt.target instanceof ImageButton)
      display_error_warning(((ImageButton)evt.target).getMessage());
    else if(evt.target instanceof ColorPalette)
      display_error_warning(((ColorPalette)evt.target).getMessage());
    else if(evt.target instanceof EasyChoice){
      if(evt.target==fontNameChoice)
        display_error_warning(((EasyChoice)evt.target).getMessage());
      else if(evt.target==fontSizeChoice)
        display_error_warning(((EasyChoice)evt.target).getMessage());
      else if(evt.target==lineWidthChoice)
        display_error_warning(((EasyChoice)evt.target).getMessage());
    }
    return true;
  }

  /**
   * Overrides awt.Component.mouseExit() to rearm the button if the mouse is
   * down, or hilite it if a special OVER image and/or border was set.
   *
   * @param   evt     The mouse enter event
   * @param   x       The mouse x position
   * @param   y       The mouse y position
   * @return  true if the event was handled
   */
  public boolean mouseExit( Event evt, int x, int y ) {
    if(evt.target instanceof ImageButton ||
       evt.target instanceof ColorPalette ||
       evt.target instanceof EasyChoice)
      display_error_warning("");
    return true;
  }

  /**
   * Overrides awt.Component.mouseDown() to rearm the button if the mouse is
   * down, or hilite it if a special OVER image and/or border was set.
   *
   * @param   evt     The mouse enter event
   * @param   x       The mouse x position
   * @param   y       The mouse y position
   * @return  true if the event was handled
   */
  public boolean mouseDown(Event event, int x, int y){
    if(event.target instanceof ColorPalette){
      if(event.shiftDown()){
        canvas.setBackgroundColor(((ColorPalette)event.target).getBackgroundColor());
        sendBackgroundColorData(canvas.getBackgroundColor());
        currentColors.setBackgroundColor(canvas.getBackgroundColor());

        if(debug) System.out.println("canvas.getBackgroundColor(): "+(canvas.getBackgroundColor()).toString());
      }
      else{
        canvas.setColor(((ColorPalette)event.target).getForegroundColor());
        currentColors.setForegroundColor(canvas.getColor());
        if(debug) System.out.println("canvas.getColor(): "+(canvas.getColor()).toString());
      }
      return true;
    }
    return false;
  }

  /**
   * Overrides awt.Component.keyDown() to rearm the button if the mouse is
   * down, or hilite it if a special OVER image and/or border was set.
   *
   * @param   evt     The mouse enter event
   * @param   x       The mouse x position
   * @param   y       The mouse y position
   * @return  true if the event was handled
   */
  public boolean keyDown(Event event, int key){
    if(debug) System.out.println("key="+key);
    if(handleKeyPress(key)) return true;
    else return false;
  }

  /**
   * Overrides awt.Component.action() to rearm the button if the mouse is
   * down, or hilite it if a special OVER image and/or border was set.
   *
   * @param   evt     The mouse enter event
   * @param   x       The mouse x position
   * @param   y       The mouse y position
   * @return  true if the event was handled
   */
  public boolean action( Event evt, Object what ){
    //if(debug) System.out.println("evt.target: "+(evt.target).toString());
    //if(debug) System.out.println("evt.paramString(): "+evt.toString());
    if(evt.target instanceof MenuItem){
      if(processFileMenu((String)evt.arg)) return true;
      if(processEditMenu((String)evt.arg)) return true;
      if(processDrawMenu((String)evt.arg)) return true;
      if(processFontMenu((String)evt.arg)) return true;
      if(processSizeMenu((String)evt.arg)) return true;
    }
    else if(evt.target instanceof Button){
      if(processButtonBar((String)evt.arg)) return true;
    }
    else if(evt.target instanceof Choice){
      if(processChoice((String)evt.arg)) return true;

    }else if(evt.target==connectImageButton){
      clickedConnectButton();
      return true;
    }else if(evt.target==disconnectImageButton){
      clickedDisconnectButton();
      return true;
    }else if(evt.target==chatImageButton){
      clickedChatButton();
      return true;
    }else if ( evt.target == boldImageButton ) {
      setBold();
      return true;
    }else if ( evt.target == italicImageButton ) {
      setItalic();
      return true;
    }else if ( evt.target == eraserImageButton ) {
      sendDeleteData(currentObjectId);
      deleteObject(currentObjectId);
      return true;
    }else if ( evt.target == textImageButton ) {
      text();
      return true;
    }else if ( evt.target == lineImageButton ) {
      line();
      return true;
    }else if ( evt.target == ovalImageButton ) {
      oval(false);
      return true;
    }else if ( evt.target == rectangleImageButton ) {
      rectangle(false);
      return true;
    }else if ( evt.target == filledOvalImageButton ) {
      oval(true);
      return true;
    }else if ( evt.target == filledRectangleImageButton ) {
      rectangle(true);
      return true;
    }else if(evt.target == freehandImageButton ) {
      freehand();
      return true;
    }
    return false;
  }

  boolean handleKeyPress(int key){
    if(key==1001 /* Delete*/ ){
      sendDeleteData(currentObjectId);
      deleteObject(currentObjectId);
      return true;
    }
    else return false;
  }

  // Method processFileMenu()
  public boolean processFileMenu(String s){

    // "New"
    if("New".equals(s)) fastConnect();


    // "Connect..."
    else if("Connect...".equals(s)) connectAs();

    // "Disconnect"
    else if("Disconnect".equals(s)) disconnectClient();

    // "Exit"
    else if("Exit".equals(s)) exit();

    return false;
  }

  // Method processEditMenu()
  public boolean processEditMenu(String s){

    // "Undo"
    if("Undo".equals(s)){
      sendDeleteData(currentObjectId);
      deleteObject(currentObjectId);

      return true;

    // "Clear"
    }else if("Clear".equals(s)){
      canvas.clear();
      currentColors.setBackgroundColor(Color.white);
      return true;
    }
    return false;
  }

  // Method processDrawMenu()
  public boolean processDrawMenu(String s){
    EasyMenu menu= menuBar.getMenu("Draw");
    CheckboxMenuItem lineItem=(CheckboxMenuItem) menu.getItem("Line");
    CheckboxMenuItem ovalItem=(CheckboxMenuItem) menu.getItem("Oval");
    CheckboxMenuItem rectangleItem=(CheckboxMenuItem) menu.getItem("Rectangle");

    if("Line".equals(s)){
      canvas.setTool(JavaDrawObject.LINE);
      lineItem.setState(true);
      ovalItem.setState(false);
      rectangleItem.setState(false);
      return true;
    }
    else if("Oval".equals(s)){
      canvas.setTool(JavaDrawObject.OVAL);
      lineItem.setState(false);
      ovalItem.setState(true);
      rectangleItem.setState(false);
      return true;
    }
    else if("Rectangle".equals(s)){
      canvas.setTool(JavaDrawObject.RECTANGLE);
      lineItem.setState(false);
      ovalItem.setState(false);
      rectangleItem.setState(true);
      return true;
    }
    return false;
  }

  // Method text()
  boolean text(){
    canvas.setTool(JavaDrawObject.TEXT);
    display_error_warning("TEXT MODE");
    return true;
  }

  // Method line()
  boolean line(){
    EasyMenu menu= menuBar.getMenu("Draw");
    CheckboxMenuItem lineItem=(CheckboxMenuItem) menu.getItem("Line");
    CheckboxMenuItem ovalItem=(CheckboxMenuItem) menu.getItem("Oval");
    CheckboxMenuItem rectangleItem=(CheckboxMenuItem) menu.getItem("Rectangle");

    canvas.setTool(JavaDrawObject.LINE);
    lineItem.setState(true);
    ovalItem.setState(false);
    rectangleItem.setState(false);
    display_error_warning("LINE MODE");
    return true;
  }

  // Method oval
  boolean oval(boolean filled){
    EasyMenu menu= menuBar.getMenu("Draw");
    CheckboxMenuItem lineItem=(CheckboxMenuItem) menu.getItem("Line");
    CheckboxMenuItem ovalItem=(CheckboxMenuItem) menu.getItem("Oval");
    CheckboxMenuItem rectangleItem=(CheckboxMenuItem) menu.getItem("Rectangle");
    if(filled) canvas.setTool(JavaDrawObject.FILLED_OVAL);
    else canvas.setTool(JavaDrawObject.OVAL);
    lineItem.setState(false);
    ovalItem.setState(true);
    rectangleItem.setState(false);
    display_error_warning("OVAL MODE");
    return true;
  }

  // Method rectangle()
  boolean rectangle(boolean filled){
    EasyMenu menu= menuBar.getMenu("Draw");
    CheckboxMenuItem lineItem=(CheckboxMenuItem) menu.getItem("Line");
    CheckboxMenuItem ovalItem=(CheckboxMenuItem) menu.getItem("Oval");
    CheckboxMenuItem rectangleItem=(CheckboxMenuItem) menu.getItem("Rectangle");
    if(filled) canvas.setTool(JavaDrawObject.FILLED_RECTANGLE);
    else canvas.setTool(JavaDrawObject.RECTANGLE);
    lineItem.setState(false);
    ovalItem.setState(false);
    rectangleItem.setState(true);
    display_error_warning("RECTANGLE MODE");
    return true;
  }

  // Method freehand()
  boolean freehand(){
    EasyMenu menu= menuBar.getMenu("Draw");
    CheckboxMenuItem lineItem=(CheckboxMenuItem) menu.getItem("Line");
    CheckboxMenuItem ovalItem=(CheckboxMenuItem) menu.getItem("Oval");
    CheckboxMenuItem rectangleItem=(CheckboxMenuItem) menu.getItem("Rectangle");
    canvas.setTool(JavaDrawObject.FREEHAND);
    lineItem.setState(false);
    ovalItem.setState(false);
    rectangleItem.setState(false);
    display_error_warning("FREEHAND MODE");
    return true;
  }


  // Method processButtonBar()
  public boolean processButtonBar(String s){

    // "CONNECT" Button
    if("CONNECT".equals(s)) clickedConnectButton();

    // "DISCONNECT" Button
    else if("DISCONNECT".equals(s)) clickedDisconnectButton();

    // "CONFERENCE" Button
    else if ("CONFERENCE".equals(s)) conference();
    return false;
  }

  // Method processViewMenu()
  public boolean processViewMenu(String s){
    if("Helvetica".equals(s)){
      boolean state;
      System.out.println("YES!!!");
      CheckboxMenuItem temp;
      temp=(CheckboxMenuItem)menuBar.getMenu("View").getItem("Show statusbar");
      state=temp.getState();
      if(state)System.out.println("on");
      else{
        System.out.println("off");
        //toolPanel.hide();
      }
      temp.setState(state);
      //System.out.println(state);
      return true;
    }
    return false;
  }

  // Method processFontMenu()
  public boolean processFontMenu(String s){
    /*for(int i=0; i<fontNames.length;++i){
      if(fontNames[i].equals(s)){
        System.out.println(fontMenuItems[i+1].toString());
        fontNameChoice.select((String)fontNames[i]);
        return true;
      }
    }*/
    return false;
  }

  // Method processSizeMenu()
  public boolean processSizeMenu(String s){
    /*for(int i=0; i<fontSizes.length;++i){
      if(fontSizes[i].equals(s)){
        System.out.println(sizeMenuItems[i+1].toString());
        fontSizeChoice.select((String)fontSizes[i]);
        return true;
      }
    }*/
    return false;
  }

  // Method processChoice()
  public boolean processChoice(String s){// Font names
    for(int i=0;i<fontNameChoice.countItems();++i){
      if(fontNameChoice.getItem(i).equals(s)){
        //currentFontName=fontNameChoice.getItem(i);
        //currentFont=new Font(currentFontName,currentFontStyle,currentFontSize);
        //canvas.setFont(currentFont);
        canvas.setFont(fontHelper.changeName(fontNameChoice.getItem(i)));
        if(debug) System.out.println("canvas.getFont(): "+(canvas.getFont()).toString());
        return true;
      }
    }
    for(int i=0;i<fontSizeChoice.countItems();++i){// Font Sizes
      if(fontSizeChoice.getItem(i).equals(s)){
        //currentFontSize=intFontSizes[i];
        //currentFont=new Font(currentFontName,currentFontStyle,currentFontSize);
        //canvas.setFont(currentFont);
        canvas.setFont(fontHelper.changeSize(i));
        if(debug) System.out.println("canvas.getFont(): "+(canvas.getFont()).toString());
        return true;
      }
    }
    for(int i=0;i<lineWidthChoice.countItems();++i){// Line Width
      if(lineWidthChoice.getItem(i).equals(s)){
        currentLineWidth=new Integer(lineWidthChoice.getItem(i)).intValue();
        canvas.setLineWidth(currentLineWidth);
        if(debug) System.out.println("currentLineWidth: "+currentLineWidth);
        return true;
      }
    }
    return false;
  }




  // Method setItalic()
  boolean setItalic(){
      int style=0;
      if((canvas.getFont()).isItalic()) style=0-Font.ITALIC;
      else style=Font.ITALIC;
      canvas.setFont(fontHelper.changeStyle(style));
      if(debug) System.out.println("canvas.getFont(): "+(canvas.getFont()).toString());
      return true;
   }

   // Method setBold()
   boolean setBold(){
      int style=0;
      if((canvas.getFont()).isBold()) style=0-Font.BOLD;
      else style=Font.BOLD;
      canvas.setFont(fontHelper.changeStyle(style));
      if(debug) System.out.println("canvas.getFont(): "+(canvas.getFont()).toString());
      return true;
   }


  // Added 2/13/97

  // Method update_dosdis()
  public void update_dosdis(DataOutputStream cdos,DataInputStream  cdis,String id_nm,String conf_nm,String ip_add){
    dos = cdos;
    dis = cdis;
    id_name = id_nm;
    conf_name = conf_nm;
    ipaddress = ip_add;
    connected = true;

    infotxt.setText(id_name+" "+conf_name+" "+" "+ipaddress);
    //disconnectButton.enable();
    //connectButton.disable();

    gdis = new getdis();
    gdis.getJavaDraw(this);
    gdis.update_dis(dis,conf_nm,id_nm);

    gdis.start();
  }

  // Method revoke_client()
  public void revoke_client(String command,String revoke_name,String revoke_time){
    if((command.length() == 0) ||(revoke_time.length() == 0) ||(revoke_time.length() == 0)){
      return;
    }
    send_data_to_server(command + " " +
                        ipaddress + " " +
                        revoke_time + " " +
                        revoke_name);
  }



  // Method disconnectClient()
  public void disconnectClient(){
    //if(debug)System.out.println("Entering clickedDisconnectButton()");
    display_error_warning("DISCONNECT MODE");
    send_data_to_server("close"+" "+ipaddress);
    if(debug)System.out.println("Disconnecting");
    if(debug)System.out.println("ipaddress: "+ipaddress);
    //disconnectClient();
    //if(debug)System.out.println("Leaving clickedDisconnectButton()");
    if(!connected) return;
    //disconnectButton.disable();
    //connectButton.enable();
    disconnectImageButton.disable();
    connectImageButton.enable();
    menuBar.getMenu("File").getItem("Connect...").enable();
    menuBar.getMenu("File").getItem("Disconnect").disable();
    menuBar.getMenu("File").getItem("New").enable();
    infotxt.setText(" ");
    gdis.update_dis(null,null,null);
    gdis.stop();
    id_name = null;
    conf_name = null;
    ipaddress = null;
    connected = false;
  }

  // Method getdis_list()
  synchronized void getdis_list(String[] str, int max_index){
    //s_list.clear();
    //s_list.update_list(str, max_index);
  }

  // Method display_error_warning()
  public void display_error_warning(String str){
    msgtxt.setText(str);
  }

  boolean clickedDisconnectButton(){
    disconnectClient();
    /*if(debug)System.out.println("Entering clickedDisconnectButton()");
    display_error_warning("DISCONNECT MODE");
    send_data_to_server("close"+" "+ipaddress);
    if(debug)System.out.println("Disconnecting");
    if(debug)System.out.println("ipaddress: "+ipaddress);
    disconnectClient();
    if(debug)System.out.println("Leaving clickedDisconnectButton()");*/
    return true;
  }

  //Method disconnect()
  /*boolean disconnect(){
    //if(debug)System.out.println("Entering clickedDisconnectButton()");
    display_error_warning("DISCONNECT MODE");
    send_data_to_server("close"+" "+ipaddress);
    if(debug)System.out.println("Disconnecting");
    if(debug)System.out.println("ipaddress: "+ipaddress);
    disconnectClient();
    //if(debug)System.out.println("Leaving clickedDisconnectButton()");
    return true;
  }*/


  boolean clickedConnectButton(){
    //if(debug)System.out.println("Entering clickedConnectButton()");
    fastConnect();
    //if(debug)System.out.println("Leaving clickedConnectButton()");
    return true;
  }

  boolean clickedChatButton(){
    //System.out.println("Va Fan !");
    chatDialog= new ChatDialog(this,"Chat",false,this,info.userName);
    chatDialog.show();
    return true;
  }

  boolean fastConnect(){
    display_error_warning("CONNECT MODE");
    ConnectInfo connectInfo = new ConnectInfo();
    connectInfo=defaultConnectSettings;
    dialog= new ConnectDialog2(this,"Create new conference",true,connectInfo);
    dialog.show();
    return true;
  }



  boolean connectAs(){
    display_error_warning("CONFERENCE MANAGEMENT MODE");

    ConnectInfo connectInfo = new ConnectInfo();
    connectInfo=defaultConnectSettings;
    //dialog= new ConnectDialog2(this,"Create new conference",true,connectInfo);
    dialog= new ConnectDialog1(this,true,connectInfo);
    dialog.show();
    return true;
  }

  //Method isConnected()
  public boolean isConnected(){
    return connected;
  }



  // Method connect()
  public void connect(Object result){
    if(debug)System.out.println("Entering connect()");
    info=(ConnectInfo)result;

    if(debug)System.out.println("=========================================");
    if(debug)System.out.println("ConnectInfo:");
    if(debug)System.out.println("userName: "+info.userName);
    if(debug)System.out.println("serverIP: "+info.serverIp);
    if(debug)System.out.println("serverPort: "+info.serverPort);
    if(debug)System.out.println("conferenceName: "+info.conferenceName);
    if(debug)System.out.println("=========================================");

    if(!connected){
      if(connect_client(info)){

        connected=true;
        update_dosdis(dos, dis, id_name, conf_name,ipaddress);
      }
      else connected=false;
    }
    if(connected){
      menuBar.getMenu("File").getItem("Disconnect").enable();
      //connectButton.disable();
      connectImageButton.disable();
      menuBar.getMenu("File").getItem("Connect...").disable();
      menuBar.getMenu("File").getItem("New").disable();

      //disconnectButton.enable();
      disconnectImageButton.enable();
    }

    //if(debug) System.out.println("connected: "+connected);
    //if(debug)System.out.println("Leaving connect()");
  }

  public boolean connect_client(ConnectInfo info){
    if(debug)System.out.println("Entering connect_client()");
    String userName;
    String serverIp;
    String serverPort;
    String conferenceName;
    String serveripaddress = null;
    int    port  =  0;
    int temp_port;
    Socket  socket  =  null;


    if(info.userName.length() == 0 || info.serverIp.length() == 0 || info.serverPort.length() == 0 || info.conferenceName.length() == 0){
      if(debug) System.out.println("ERROR: connection");
      msgtxt.setText("ID OR IP Address OR port OR Conference is Empty");
      return false;
    }
    else{
      serveripaddress = info.serverIp;
      port = new Integer(info.serverPort).intValue();
      id_name = info.userName;
      conf_name = info.conferenceName;
      if(debug) System.out.println("serveripaddress: "+serveripaddress);
      if(debug) System.out.println("port: "+port);
      if(debug) System.out.println("id_name: "+info.userName);
      if(debug) System.out.println("conf_name: "+conf_name);
    }

    try{
      socket = new Socket(serveripaddress, port);
      System.out.println("Socket" + socket);
      dos = new DataOutputStream(socket.getOutputStream());
      dis = new DataInputStream(socket.getInputStream());
      temp_port = socket.getLocalPort();
      System.out.println("funkar: " +serveripaddress);
    }
    catch(IOException  e){
      connected = false;
      System.out.println("Error:  IO  Exception");
      msgtxt.setText("Error: Cannot Connect to Server");
      return false;
    }

    try{
      InetAddress add = null;
      add = InetAddress.getLocalHost();

      String ipadd_str = add.toString();
      StringTokenizer inputbufferST = new StringTokenizer(ipadd_str,"/");
      String ipadd_str2 = inputbufferST.nextToken();
      ipadd_str2 = inputbufferST.nextToken();
      String ipadd = ipadd_str2+":"+Integer.toString(temp_port);
      ipaddress = ipadd;
    }
    catch(UnknownHostException e){
      connected = false;
      return false;
    }

    try{
      dos.writeBytes(conf_name + " " +
                       id_name + " " +
                       "Connect" + " " +
                       ipaddress + " " +
                       "0" + " " +
                       "\n");
      dos.flush();
      connected = true;
      msgtxt.setText("Connected To Server");
      return true;
    }
    catch(IOException  e){
      connected = false;
      if(debug)System.out.println("Error:  IO  Exception");
      msgtxt.setText("Error: Cannot Connect to Server");
      return false;
    }
  //if(debug)System.out.println("Leaving connect_client()");
  }




  // Method conference()
  boolean conference(){
    display_error_warning("CONFERENCE MANAGEMENT MODE");
    l_diag.revoketxt.setText(" ");
    l_diag.show();

    //ConnectInfo connectInfo = new ConnectInfo();
    //connectInfo=defaultConnectSettings;
    //dialog= new ConnectDialog2(this,"Create new conference",true,connectInfo);
    //dialog= new ConnectDialog1(this,true);
    //dialog.show();
    return true;
  }

  public void processResult(Dialog source, Object result){
    if(debug)System.out.println("Entering processResult()");
    if(source instanceof ConnectDialog2) connect(result);
    if(debug)System.out.println("Leaving processResult()");

  }


  void getDefaultConnectSettings(){
    InetAddress localAddress=null;
    defaultConnectSettings.userName="";
    try{
      localAddress = InetAddress.getLocalHost();
    }
    catch(UnknownHostException ex){
      //error("Error getting local host information");
    }
    String hostName=localAddress.getHostName();
    defaultConnectSettings.serverIp=hostName;
    defaultConnectSettings.serverPort="9005";
    defaultConnectSettings.conferenceName="Conference1";
  }

  synchronized boolean getdis_pen(int x1,
                                  int y1,
                                  int x2,
                                  int y2,
                                  int objectType,
                                  int objectId,
                                  int objectColor,
                                  int fill)
  {
    canvas.addObject(x1,y1,x2,y2,objectType,objectId,objectColor,fill);
    return true;
  }

  synchronized boolean getdis_text(int x,
                                   int y,
                                   int w,
                                   int h,
                                   int size,
                                   int style,
                                   String name,
                                   int type,
                                   int id,
                                   int charId,
                                   int key,
                                   int rgb)
  {
    if(debug)System.out.println("text received");
    canvas.addTextObject(x,y,w,h,size,style,name,type,id,charId,key,rgb);
    return true;
  }

  synchronized boolean getdis_delete(int id){
    deleteRemoteObject(id);
    return true;
  }

  synchronized boolean getdis_background(int color){
    canvas.setBackgroundColor(new Color(color));
    currentColors.setBackgroundColor(new Color(color));
    return true;
  }

  synchronized boolean getdis_ctrlChar(int id,int charId,int key){
    if(debug)System.out.println("deleteChar received");
    canvas.deleteTextObject(id,charId,key);
    return true;
  }

  synchronized boolean getdis_chat(String name,String msg){
    if(debug)System.out.println("chat received");
    if(chatDialog!=null){
      chatDialog.printRemote(name,msg);
    }
    return true;
  }
  public void sendBackgroundColorData(Color color){
	  if(!connected) return;
    int rgb=(int)color.getRGB();
    display_error_warning("Sending COLOR INFO");
    send_data_to_server("B"+" "+rgb);
  }

  public void sendDeleteData(int id){
    if(debug) System.out.println("Entering: sendDeleteData()");
	  if(!connected) return;
    display_error_warning("Sending DELETE INFO");
    send_data_to_server("D"+" "+id);
    if(debug) System.out.println("Leaving: sendDeleteData()");
  }

  public void sendPenData(int x1,int y1,int x2,int y2,int objectType,int objectId,int objectColor, int fill){
	  if(debug) System.out.println("Entering: sendPenData()");
	  if(!connected) return;
    display_error_warning("Sending PEN INFO");
    send_data_to_server("P"+" "+x1+" "+y1+" "+x2+" "+y2+" "+objectType+" "+" "+objectId+" "+objectColor+" "+fill);
    //dos.flush();
    if(debug) System.out.println("Leaving: sendPenData()");
  }

  public void sendTextData(int x,int y,int w,int h,int size,int style,String name,int objectType,int objectId,int charId,int key,int objectColor){
	  if(debug) System.out.println("Entering: sendTextData()");
	  if(!connected) return;
    display_error_warning("Sending TEXT INFO");
    send_data_to_server("T"+" "+x+" "+y+" "+w+" "+h+" "+size+" "+style+" "+name+" "+objectType+" "+objectId+" "+charId+" "+key+" "+objectColor);
    //dos.flush();
    if(debug) System.out.println("Leaving: sendTextData()");
  }

  public void sendCtrlCharData(int id, int charId, int key){
	if(debug) System.out.println("Entering: sendDelCharData()");
    send_data_to_server("C"+" "+id+" "+charId+" "+key);
    if(debug) System.out.println("Leaving: sendDelCharData()");
  }

   public void sendChatData(String msg){
	if(debug) System.out.println("Entering: sendChatData()");
      String name=new String(info.userName);
      if(name==null) name="local";
      //String message=new String(name+": "+msg+"\n");

    send_data_to_server("H"+" "+name+" "+msg);
    System.out.println("Leaving: sendChatData()"+msg);
  }

  public void send_data_to_server(String str){
    if(debug) System.out.println("Entering: send_data_to_server()");
    if(!connected ||(str.length() == 0) ||(id_name.length() == 0) ||(conf_name.length() == 0) ||(dos == null)){
      return;
    }
    try{
      dos.writeBytes(conf_name+" "+id_name  +" "+str+"\n");
      dos.flush();
    }
    catch(IOException e){
      System.out.println("ERROR: Writing");
    }
  }

  public int getObjectId(){
    return currentObjectId++;
  }

  public void decrementObjectId(){
    currentObjectId--;
  }

  public int getLastObjectId(){
    return currentObjectId;
  }

  void deleteObject(int id){
    display_error_warning("DELETE MODE");
    if(canvas.deleteObject(id-1)) currentObjectId--;
  }

  void deleteRemoteObject(int id){
    if(canvas.deleteObject(id-1)); //currentObjectId--;
  }

  // Method exit()
  boolean exit(){
    if(!connected){
      dispose();
      hide();
      System.exit(0);
    }else stillConnected.show();
    return true;
  }



  // Method setupMenuBar()
  void setupMenuBar(){
    menuBar= new EasyMenuBar(menuItems);
  }

  // Method setupFonts()
  void setupFonts(){
    menuItems[2][1]=fontHelper.getFontMenuItems();
    menuItems[2][2]=fontHelper.getStyleMenuItems();
    menuItems[2][3]=fontHelper.getSizeMenuItems();
  }

  // Method setupButtonBar()
  void setupButtonBar(){

    Panel topPanel =new Panel();
    Panel buttonPanel1= new Panel();
    Panel buttonPanel2= new Panel();
    topPanel.setLayout(new GridLayout(2,1));
    buttonPanel1.setLayout(new FlowLayout(FlowLayout.LEFT));
    buttonPanel2.setLayout(new FlowLayout(FlowLayout.LEFT));

    Image image;
    buttonPanel1.add(newImageButton=new ImageButton(image=toolkit.getImage("./imgbtn/new1.gif"),"New"));
    buttonPanel1.add(openImageButton=new ImageButton(image=toolkit.getImage("./imgbtn/open1.gif"),"Open"));
    buttonPanel1.add(saveImageButton=new ImageButton(image=toolkit.getImage("./imgbtn/save1.gif"),"Save"));
    buttonPanel1.add(new Label(" "));
    buttonPanel1.add(chatImageButton=new ImageButton(image=toolkit.getImage("./imgbtn/chat1.gif"),"Chat"));
    buttonPanel1.add(new Label(" "));
    buttonPanel1.add(connectImageButton=new ImageButton(image=toolkit.getImage("./imgbtn/connect2.gif"),"Connect"));
    buttonPanel1.add(disconnectImageButton=new ImageButton(image=toolkit.getImage("./imgbtn/disconnect1.gif"),"Disconnect"));

    buttonPanel1.add((conferenceButton =new Button("CONFERENCE")));

    buttonPanel2.add((fontNameChoice =new EasyChoice(fontHelper.getFontNames(),"Font")));
    buttonPanel2.add((fontSizeChoice=new EasyChoice(fontHelper.getFontSizes(),"Size")));


    buttonPanel2.add(boldImageButton=new ImageButton(image=toolkit.getImage("./imgbtn/bold1.gif"),"Bold Font"));
    buttonPanel2.add(italicImageButton=new ImageButton(image=toolkit.getImage("./imgbtn/italic1.gif"),"Italic Font"));

    buttonPanel2.add(new Label("    Line width"));
    buttonPanel2.add((lineWidthChoice=new EasyChoice(lineWidth,"Line Width")));

    topPanel.add(buttonPanel1);
    topPanel.add(buttonPanel2);
    add("North",topPanel);
  }

  // Method setupToolButtonBar()
  void setupToolButtonBar(){
    //EasyChoice fontNameChoice;
    //EasyChoice fontSizeChoice;

    Panel toolPanel =new Panel();
    Panel bPanel1= new Panel();
    Panel bPanel2= new Panel();
    Panel bPanel3= new Panel();
    toolPanel.setLayout(new BorderLayout());
    bPanel1.setLayout(new GridLayout(4,2));
    bPanel2.setLayout(new FlowLayout(FlowLayout.LEFT));
    bPanel3.setLayout(new FlowLayout(FlowLayout.LEFT));

    Image image;
    eraserImageButton = new ImageButton( image=toolkit.getImage("./imgbtn/eraser1.gif"),"Eraser" );
    textImageButton = new ImageButton( image=toolkit.getImage("./imgbtn/text1.gif"),"Text Tool" );
    lineImageButton = new ImageButton( image=toolkit.getImage("./imgbtn/line1.gif"),"Line Tool" );
    ovalImageButton = new ImageButton( image=toolkit.getImage("./imgbtn/oval1.gif"),"Oval Tool");
    rectangleImageButton = new ImageButton( image=toolkit.getImage("./imgbtn/rectangle1.gif"),"Rectangle Tool" );
    filledOvalImageButton =new ImageButton(image=toolkit.getImage("./imgbtn/oval2.gif"),"Solid Oval Tool");
    filledRectangleImageButton =new ImageButton(image=toolkit.getImage("./imgbtn/rectangle2.gif"),"Solid Rectangle Tool");
    freehandImageButton =new ImageButton(image=toolkit.getImage("./imgbtn/freehand1.gif"),"Freehand Tool");
    bPanel1.add( eraserImageButton );
    bPanel1.add( textImageButton );
    bPanel1.add( lineImageButton );
    bPanel1.add( ovalImageButton );
    bPanel1.add( rectangleImageButton );
    bPanel1.add( filledOvalImageButton );
    bPanel1.add( filledRectangleImageButton );
    bPanel1.add( freehandImageButton );

    bPanel2.add(new ColorPalette(3,"Color Palette"));
    bPanel3.add(currentColors=new CurrentColors(canvas.getColor(),canvas.getBackgroundColor()));
    toolPanel.add("North",bPanel1);
    toolPanel.add("West",bPanel2);
    toolPanel.add("South",bPanel3);
    add("West",toolPanel);
  }

  /**
   * Method setupStatusBar()
   * Adds a statusbar at the botom of the Zodega window.
   */

  void setupStatusBar(){
    Panel statusBar = new Panel();
    statusBar.setLayout(new FlowLayout(FlowLayout.LEFT));
    //msglbl = new Label("Message:");
    //statusBar.add(msglbl);
    msgtxt = new TextField(25);
    msgtxt.setEditable(false);
    statusBar.add(msgtxt);
    infotxt = new TextField(30);
    infotxt.setEditable(false);
    statusBar.add(infotxt);
    add("South", statusBar);
  }

  // Method init()
  public  void  init(){
    //c_diag = new ConnectDialog(this);
    //c_diag.getJavaDraw(this);
    l_diag = new list_dialog(this);
    l_diag.getJavaDraw(this);
	  stillConnected = new StillConnected(this);
    s_list = l_diag.get_sl();
   /*     dim  =  size();
        wdth  =  dim.width;
        hght  =  dim.height;

        l_lastx  =  0;
        l_lasty  =  0;
        l_penthickness  =  0;
        l_erasethickness  =  5;
        l_lasttype  =  0;
        l_color  =  0;
        l_keyx   = 0;
        l_keyy   = 0;
        l_name   = "TimesRoman";
        l_style  = 'P';
        l_size   = 16;

        r_lastx  =  0;
        r_lasty  =  0;
        r_penthickness  =  0;
        r_erasethickness  =  5;
        r_lasttype  =  0;
        r_color  =  0;
        r_name   = "TimesRoman";
        r_style  = 'P';
        r_size   = 16;

        //Erasex  =  20;
        //Erasey  =  h  -  25;
        gotFirstPoint  =  false;
        updateDisplay  =  true;*/
        if(debug)System.out.println("=========================================");
        if(debug) System.out.println("Initialization of Zodega");

        //toolkit=getToolkit();
        //fontHelper=new FontHelper(toolkit);
        //canvas=new ExtendedCanvas(this,JavaDrawObject.LINE);


        menuBar.getMenu("File").getItem("Disconnect").disable();
        if(debug) System.out.println("Disabeling: File->Discconect");
        //disconnectButton.disable();
        disconnectImageButton.disable();
        if(debug) System.out.println("Disabeling: disconnectImageButton");
        //defaultFont=getFont();
        //if(debug) System.out.println("Default font: "+defaultFont.toString());
        //currentFont=defaultFont;
        //if(debug) System.out.println("Current font: "+currentFont.toString());
        canvas.setDefaultFont(fontHelper.getDefaultFont());
        if(debug) System.out.println("Canvas font: "+(canvas.getFont()).toString());
        //currentFontName=currentFont.getName();
        //if(debug) System.out.println("currentFontName: "+currentFontName);
        //currentFontSize=currentFont.getSize();
        //if(debug) System.out.println("currentFontSize: "+currentFontSize);
        //currentFontStyle=currentFont.getStyle();
        //if(debug) System.out.println("currentFontStyle: "+currentFontStyle);
        String temp=new String();

        fontNameChoice.select((String)(canvas.getDefaultFont()).getName());
        if(debug) System.out.println("Default font selected in: fontNameChoice");
        //if(debug) System.out.println("(canvas.getDefaultFont()).getName(): "+(canvas.getDefaultFont()).getName());

        fontSizeChoice.select(temp.valueOf((canvas.getDefaultFont()).getSize()));
        if(debug) System.out.println("Default size selected in: fontSizeChoice");

        /*EasyMenu menu= menuBar.getMenu("Text");
        for(int i=0;i<menu.countItems();++i){
          MenuItem item=menu.getItem(i);
          System.out.println(item.getLabel());
        }*/

        /*CheckboxMenuItem fontItem=(CheckboxMenuItem) menu.getItem("Font");*/
        //System.out.println(fontItem.getLabel());
        //CheckboxMenuItem ovalItem=(CheckboxMenuItem) menu.getItem("Oval");
        //CheckboxMenuItem rectangleItem=(CheckboxMenuItem) menu.getItem("Rectangle");
        //EasyMenu menu2= menuBar.getMenu("Text");
        //CheckboxMenuItem lineItem=(CheckboxMenuItem) menu2.getItem("sex");
        //lineItem.setState(false);
        //CheckboxMenuItem ovalItem=(CheckboxMenuItem) menu.getItem("Oval");
        //CheckboxMenuItem rectangleItem=(CheckboxMenuItem) menu.getItem("Rectangle");

        if(debug) System.out.println("Printing: LINE MODE to status bar.");
        display_error_warning("LINE MODE");

        getDefaultConnectSettings();
        if(debug) System.out.println("\ndefaultConnectSettings:");
        if(debug) System.out.println("userName: "+defaultConnectSettings.userName);
        if(debug) System.out.println("serverIP: "+defaultConnectSettings.serverIp);
        if(debug) System.out.println("serverPort: "+defaultConnectSettings.serverPort);
        if(debug) System.out.println("conferenceName: "+defaultConnectSettings.conferenceName+"\n");

        defaultObjectId= Math.abs((int)new Random().nextInt());
        currentObjectId=defaultObjectId;
        if(debug) System.out.println("defaultObjectId= "+defaultObjectId);
        if(debug) System.out.println("currentObjectId= "+currentObjectId);

        currentLineWidth=defaultLineWidth;
        if(debug) System.out.println("currentLineWidth: "+currentLineWidth);

        canvas.requestFocus();
        if(debug) System.out.println("Focus set on canvas");

        if(debug) System.out.println("End of Zodega init.");


        //str[0]="temp".toString();

        if(debug)System.out.println("=========================================");
    }  //init()




  // Variable Declarations


  // Debug
  public boolean debug=false;


  //Text Fields
  TextField msgtxt;
  TextField infotxt;

  //Labels
  Label iplbl;
  Label portlbl;
  Label conflbl;
  Label msglbl;

  //Buttons
  Button penButton;
  Button textButton;
  Button eraseButton;
  Button cleanButton;
  //Button connectButton;
  //Button disconnectButton;
  Button conferenceButton;

  // Image Buttons

  ImageButton newImageButton;
  ImageButton openImageButton;
  ImageButton saveImageButton;

  ImageButton chatImageButton;
  ImageButton printImageButton;

  ImageButton connectImageButton;
  ImageButton disconnectImageButton;

  ImageButton boldImageButton;
  ImageButton italicImageButton;

  ImageButton eraserImageButton;
  ImageButton textImageButton;
  ImageButton lineImageButton;
  ImageButton ovalImageButton;
  ImageButton rectangleImageButton;
  ImageButton filledOvalImageButton;
  ImageButton filledRectangleImageButton;
  ImageButton freehandImageButton;


  // Choices
  EasyChoice fontNameChoice;
  EasyChoice fontSizeChoice;
  EasyChoice lineWidthChoice;

  // Dialog
  Dialog dialog;

  // Screen size properties
  int screenWidth=650;
  int screenHeight=500;


  // Fonts

  String temp1[]={"Font"};


  Object menuItems[][]={
    {"File","New","Connect...","Disconnect","-","Exit"},
    {"Edit","Undo","Clear"},
    {"Text",temp1,temp1,temp1},
    {"Draw","+Line","-Oval","-Rectangle"},
    {"View","+Show toolbar","+Show statusbar"},
    {"Help","About..."},
    };

  int defaultObjectId;
  int currentObjectId;

  int defaultLineWidth=1;
  int currentLineWidth;
  String lineWidth[]={"1","2","3","4","5","6","8"};

  EasyMenuBar menuBar;
  CurrentColors currentColors;
  Toolkit toolkit=getToolkit();
  FontHelper fontHelper=new FontHelper(toolkit);
  ExtendedCanvas canvas=new ExtendedCanvas(this,JavaDrawObject.LINE);
  ConnectInfo defaultConnectSettings=new ConnectInfo();
  ChatDialog chatDialog;
  ConnectInfo info;

  // unsorted defenitions added 2/13/97

  // Network and Socket variables
  //public  Socket  socket  =  null;
  DataInputStream   dis  =  null;
  DataOutputStream  dos  =  null;
  boolean text_mode = false;
  int width_increment = 0;
  //ConnectDialog	c_diag;
  StillConnected     stillConnected;
  list_dialog		l_diag;


  show_list 		s_list;
  String 		id_name;
  String 		conf_name;
  String		ipaddress  =  null;
  protected boolean connected = false;
  getdis		gdis;





}// end of Zodega class

