package extended;
//ExtendedCanvas.java

import java.awt.*;
import java.util.Vector;
import java.lang.*;

public class ExtendedCanvas extends Canvas {

  public static int INITIAL_SIZE=100;
  public static int INCREMENT=100;
  int tool = TwoPointObject.LINE;
  Vector objects = new Vector(INITIAL_SIZE,INCREMENT);
  TwoPointObject current=null;
  TwoPointObject remoteObject=null;
  TwoPointObject point=new TwoPointObject();
  boolean newObject = false;
  Color objColor=Color.black;
  Color defaultBackgroundColor=Color.white;
  Color backgroundColor=defaultBackgroundColor;
  int objLineWidth=1;
  ResultProcessor owner;
  int freehandId=0;
  boolean debug=false;
  int firstElement=0;
  int lastElement=0;
  Font objFont;
  Font defaultFont;





  public ExtendedCanvas(ResultProcessor temp,int toolType) {
    super();
    tool = toolType;
    owner =temp;
    setBackground(backgroundColor);

  }


  public void setTool(int toolType) {
    tool = toolType;
  }

  public void setColor(Color color){
    objColor=color;
  }

  public Color getColor(){
    return objColor;
  }

  public void setBackgroundColor(Color color){
    backgroundColor=color;
    setBackground(backgroundColor);
    repaint();
  }

  public Color getBackgroundColor(){
    return backgroundColor;
  }

  public void setLineWidth(int width){
    objLineWidth=width;
  }

  public int getLineWidth(){
    return objLineWidth;
  }

  public void clear() {
    objects.removeAllElements();
    setBackground(defaultBackgroundColor);
    repaint();
  }

  public void setFont(Font font){
    objFont=font;
  }

  public Font getFont(){
    return objFont;
  }

  public void setDefaultFont(Font font){
    defaultFont=font;
    objFont=defaultFont;
  }

  public Font getDefaultFont(){
    return defaultFont;
  }


  /**
   * Delete  id TwoPointObject on local canvas.
   *
   * @param   id      The objects unique identifier
   *
   */
  public boolean deleteObject(int id){
    boolean deleted=false;
    if(debug) System.out.println("id of object to remove: "+id);
    if(objects.isEmpty()){
      if(debug) System.out.println("objects is empty");
      return false;
    }
    else{
      if(debug) System.out.println("objects.size()="+objects.size());
      int x1=0,y1=0,x2=0,y2=0,width=0,type=0;
      for(int i=objects.size()-1;i>=0;i--){
        if(((TwoPointObject)objects.elementAt(i)).getId()==id){
          x1=((TwoPointObject)objects.elementAt(i)).startX;
          y1=((TwoPointObject)objects.elementAt(i)).startY;
          x2=((TwoPointObject)objects.elementAt(i)).endX;
          y2=((TwoPointObject)objects.elementAt(i)).endY;
          width=((TwoPointObject)objects.elementAt(i)).objectLineWidth;
          tool=((TwoPointObject)objects.elementAt(i)).type;
          objects.removeElementAt(i);
          deleted=true;
          if(type==TwoPointObject.FREEHAND)
            repaint(x1,y1,width,width);
          else if(type==TwoPointObject.LINE)
            repaint(Math.min(x1,x2)-width,Math.min(y1,y2)-width,Math.abs(x2-x1)+2*width,Math.abs(y2-y1)+2*width);
          else repaint(x1,y1,x2,y2);
        }
      }
      return deleted;
    }

  }

  /**
   * Add a remote TwoPointObject to local canvas.
   *
   * @param   x1      The objects upper right corner
   * @param   y1      The objects upper right corner
   * @param   x2      Second x coordinat, used as lower left corner or width
   * @param   y2      Second y coordinat, used as lower left corner or width
   * @param   tool    Specifies object type
   * @param   id      The objects unique identifier
   * @param   RGB     The objects color
   * @param   width   The objects line width
   *
   */
  public void addObject(int x1, int y1, int x2, int y2, int tool,int id, int RGB, int width){
    remoteObject = new TwoPointObject(tool,id,x1,y1,x2,y2,width);
    Color color=new Color(RGB);
    remoteObject.setColor(color);
    remoteObject.setId(id);
    remoteObject.setLineWidth(width);
    if(debug) System.out.println("remoteObjectId: "+remoteObject.getId());
    objects.addElement(remoteObject);
    repaint(remoteObject.startX,remoteObject.startY,remoteObject.endX,remoteObject.endY);
  }

  /**
   * Overrides awt.Component.mouseDown()
   *
   * @param   event   The mouse down event
   * @param   x       The mouse x position
   * @param   y       The mouse y position
   * @return  true if the event was handled
   */
  public boolean mouseDown(Event event,int x,int y) {
    if(tool==TwoPointObject.FREEHAND){
      freehandId=owner.getObjectId();
      firstElement=objects.size();
      if(debug) System.out.println("firstElement="+firstElement);
    }
    else{
      current = new TwoPointObject(tool,x,y);
      current.setColor(objColor);
      current.setId(owner.getObjectId());
      current.setLineWidth(objLineWidth);
      if(debug) System.out.println("currentId: "+current.getId());
      newObject = true;
    }
    return true;
  }

  /**
   * Overrides awt.Component.mouseUp()
   *
   * @param   evt     The mouse up event
   * @param   x       The mouse x position
   * @param   y       The mouse y position
   * @return  true if the event was handled
   */
  public boolean mouseUp(Event event,int x,int y) {
    if(tool==TwoPointObject.FREEHAND){
      lastElement=objects.size()-1;
      if(debug) System.out.println("lastElement="+lastElement);
      for(int i=firstElement;i<lastElement;i++){
        sendObject((TwoPointObject)objects.elementAt(i));
      }
    }
    else if(newObject){
      objects.addElement(current);
      sendObject(current);
      newObject = false;
    }
    return true;
  }

  /**
   * Overrides awt.Component.mouseDrag()
   *
   * @param   evt     The mouse up event
   * @param   x       The mouse x position
   * @param   y       The mouse y position
   * @return  true if the event was handled
   */
  public boolean mouseDrag(Event event,int x,int y) {
    if(tool==TwoPointObject.FREEHAND){
      if((float)Math.abs(point.startX-x)>=(float)objLineWidth*.75 || (float)Math.abs(point.startY-y)>=(float)objLineWidth*.75){
        point = new TwoPointObject(tool,x,y);
        point.setColor(objColor);
        point.setId(freehandId);
        point.setLineWidth(objLineWidth);
        if(debug) System.out.println("pointId: "+point.getId());
        objects.addElement(point);
        repaint(point.startX,point.startY,point.objectLineWidth,point.objectLineWidth);
      }
    }
    else if(newObject) {
      int oldX = current.endX;
      int oldY = current.endY;
      if(tool != TwoPointObject.LINE) {
        if((event.modifiers & Event.SHIFT_MASK /* +Shift */)!=0){
          int difference=Math.abs(x-current.startX)-Math.abs(y-current.startY);
          if(difference>=0)y+=difference;
          else x+=Math.abs(difference);
        }
        if(x > current.startX) current.endX = x;
        if(y > current.startY) current.endY = y;
        int width = Math.max(oldX,current.endX) - current.startX + 1;
        int height = Math.max(oldY,current.endY) - current.startY + 1;
        repaint(current.startX,current.startY,width,height);
      }
      else{
        current.endX = x;
        current.endY = y;
        int startX = Math.min(Math.min(current.startX,current.endX),oldX);
        int startY = Math.min(Math.min(current.startY,current.endY),oldY);
        int endX = Math.max(Math.max(current.startX,current.endX),oldX);
        int endY = Math.max(Math.max(current.startY,current.endY),oldY);
        repaint(startX-objLineWidth,startY-objLineWidth,endX-startX+objLineWidth*2,endY-startY+objLineWidth*2);
      }
    }
    return true;
  }

  /**
   * Overrides awt.Component.paint() to paint the current TwoPointObject.
   *
   * @param     obj     The TwoPointObject to send
   */
  void sendObject(TwoPointObject obj){
    if(owner.isConnected()){
      owner.sendPenData(obj.startX,obj.startY,obj.endX,obj.endY,obj.type,obj.getId(),(int)(obj.getColor()).getRGB(),obj.getLineWidth());
    }
  }

  /**
   * Overrides awt.Component.paint() to paint the current TwoPointObject.
   *
   * @param     g     The Graphics in which to draw
   */
  public void paint(Graphics g) {
    int numObjects = objects.size();
    for(int i=0;i<numObjects;++i) {
      TwoPointObject obj = (TwoPointObject) objects.elementAt(i);
      obj.draw(g);
    }
    if(newObject) current.draw(g);
  }


}// End of class ExtendedCanvas
